####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################

import sys, time, datetime, types, re
from Number import number
from picalo import parse_value_to_type
  
# global functions (defined in this file)
__all__ = [
  'Date', 
  'DateTime',
  'DateFormat',
  'DateTimeFormat',
  'TimeDelta',
  'DateDelta',
]


class DateFormatInfo:
  '''A simple class to represent information about a date format'''
  def __init__(self, regex, format):
    self.regex = regex
    self.format = format


# these are all the date formats we search through to autodiscover the format a date is in.  
# this is mostly used in Column.parse_value()
DATE_FORMATS = [
  [ '\d{4}-\d{1,2}-\d{1,2}',   '%Y-%m-%d' ],   # 2010-01-31
  [ '\d{2}-\d{1,2}-\d{1,2}',   '%y-%m-%d' ],   # 10-01-31
  [ '\d{1,2}-\d{1,2}-\d{4}',   '%m-%d-%Y' ],   # 01-31-2010

  [ '\d{4}\.\d{1,2}\.\d{1,2}', '%Y.%m.%d' ],   # 2010.01.31
  [ '\d{2}\.\d{1,2}\.\d{1,2}', '%y.%m.%d' ],   # 10.01.31
  [ '\d{1,2}\.\d{1,2}\.\d{4}', '%m.%d.%Y' ],   # 01.31.2010

  [ '\d{4} \d{1,2} \d{1,2}',   '%Y %m %d' ],   # 2010 01 31
  [ '\d{2} \d{1,2} \d{1,2}',   '%y %m %d' ],   # 10 01 31
  [ '\d{1,2} \d{1,2} \d{4}',   '%m %d %Y' ],   # 01 31 2010

  [ '\d{4}\/\d{1,2}\/\d{1,2}', '%Y/%m/%d' ],   # 2010/01/31
  [ '\d{2}\/\d{1,2}\/\d{1,2}', '%y/%m/%d' ],   # 10/01/31
  [ '\d{1,2}\/\d{1,2}\/\d{4}', '%m/%d/%Y' ],   # 01/31/2010

  [ '\d{4}-\w{3}-\d{1,2}',     '%Y-%b-%d' ],   # 2010-Jan-31
  [ '\d{2}-\w{3}-\d{1,2}',     '%y-%b-%d' ],   # 10-Jan-31
  [ '\d{4}-\w{3,}-\d{1,2}',    '%Y-%B-%d' ],   # 2010-January-31
  [ '\d{2}-\w{3,}-\d{1,2}',    '%y-%B-%d' ],   # 10-January-31

  [ '\d{1,2} \w{3} \d{4}',     '%d %b %Y' ],   # 25 Jan 2005
  [ '\d{1,2} \w{3} \d{2}',     '%d %b %y' ],   # 25 Jan 05
  [ '\d{1,2} \w{3,} \d{4}',    '%d %B %Y' ],   # 25 January 2005
  [ '\d{1,2} \w{3,} \d{2}',    '%d %B %y' ],   # 25 January 05
  [ '\w{3} \d{1,2}, \d{4}',    '%b %d, %Y' ],  # Jan 25, 2005
  [ '\w{3} \d{1,2}, \d{2}',    '%b %d, %Y' ],  # Jan 25, 05
  [ '\w{3,} \d{1,2}, \d{4}',   '%B %d, %Y' ],  # January 25, 2005
  [ '\w{3,} \d{1,2}, \d{2}',   '%B %d, %Y' ],  # January 25, 05
]
RE_DATE_FORMATS = []  # used in the TableProperties dialog
RE_DATETIME_FORMATS = []
# first add the native datetime format
NATIVE_FORMAT_PARSER = re.compile('^(\d{4})-(\d{2})-(\d{2}) +(\d{2}):(\d{2}):(\d{2}).(\d{2,7})$')
RE_DATETIME_FORMATS.append(DateFormatInfo(re.compile('^\d{4}-\d{2}-\d{2} +\d{2}:\d{2}:\d{2}.\d+$'), ''))  
# now add some other formats
for repattern, timepattern in DATE_FORMATS:  # make reg expressions for date plus different forms of time and zone (makes a lot of patterns!)
  RE_DATE_FORMATS.append(DateFormatInfo(re.compile('^' + repattern + "$"), timepattern))   
  RE_DATETIME_FORMATS.append(DateFormatInfo(re.compile('^' + repattern + "$"), timepattern))
  RE_DATETIME_FORMATS.append(DateFormatInfo(re.compile('^' + repattern + ' +\d{1,2}:\d{1,2}:\d{1,2}$'), timepattern + ' %H:%M:%S'))
  RE_DATETIME_FORMATS.append(DateFormatInfo(re.compile('^' + repattern + ' +\d{1,2}:\d{1,2}:\d{1,2}(AM|PM|am|pm)$'), timepattern + ' %I:%M:%S%p'))
  RE_DATETIME_FORMATS.append(DateFormatInfo(re.compile('^' + repattern + ' +\d{1,2}:\d{1,2}:\d{1,2} +(AM|PM|am|pm)$'), timepattern + ' %I:%M:%S %p'))


#####################################################
###   Date arithmetic helper method

def __coerce_value__(other):
  '''Function that allows Date arithmetic with standard Python types'''
  if isinstance(other, (int, long, float, number)):
    return TimeDelta(other)
  elif isinstance(other, (unicode, str)):
    return TimeDelta(parse_value_to_type(other, float))
  return other
    
    


####################################################
###  Standard DateTime type used in Picalo


def _parse(klass, subklass, stdate, format):
  '''Internal function to initialize a date into a date or datetime object'''
  if format:
     parts = time.strptime(stdate, format)
     if klass == datetime.datetime:
       return klass.__new__(subklass, parts[0], parts[1], parts[2], parts[3], parts[4], parts[5])
     elif klass == datetime.date:
       return klass.__new__(subklass, parts[0], parts[1], parts[2])
     raise SystemError, 'Invalid klass used in DateTime._parse'
  # special code to parse the native datetime format, since strptime can't do it
  m = NATIVE_FORMAT_PARSER.search(stdate)  # milliseconds
  if m and klass == datetime.datetime:
    msec = m.group(7)
    microsecs = int(msec + ('0' * (6-len(msec))))
    return klass.__new__(subklass, int(m.group(1)), int(m.group(2)), int(m.group(3)), int(m.group(4)), int(m.group(5)), int(m.group(6)), microsecs)
  elif m and klass == datetime.date:
    return klass.__new__(subklass, int(m.group(1)), int(m.group(2)), int(m.group(3)))
  raise ValueError, 'Value is not one of the known date formats'


last_re_date_used = 0  # see the function below for this
def _create(klass, subklass, *args, **kargs):
  '''Internal function to create a date or time'''
  # these if statements in the function are in order of the most-used to least used, for a speed increase
  global last_re_date_used
  
  # check if we have exactly one argument
  if len(args) == 1:
    # see if we already have a datetime
    if isinstance(args[0], (datetime.datetime, datetime.date)):
      if klass == datetime.datetime and type(args[0]) in (DateTime, datetime.datetime):  # if already the right type, just return it (btw, isinstance doesn't work)
        return args[0]
      return klass.__new__(subklass, args[0].year, args[0].month, args[0].day)  
      
    # check if we have a single string for the argument
    if isinstance(args[0], types.StringTypes):
      # first check the last one we used, since we often parse thousands of rows with the same format in each row
      match = RE_DATETIME_FORMATS[last_re_date_used].regex.match(args[0])
      if match:
        return _parse(klass, subklass, args[0], RE_DATETIME_FORMATS[last_re_date_used].format)
      # the last one didn't match, so go through the entire list of patterns
      for i in range(len(RE_DATETIME_FORMATS)):
        match = RE_DATETIME_FORMATS[i].regex.match(args[0])
        if match:
          last_re_date_used = i
          return _parse(klass, subklass, args[0], RE_DATETIME_FORMATS[last_re_date_used].format)
      raise ValueError, 'Value is not in one of the known date formats: ' + args[0] + '.  Use Date(value, format) or DateTime(value, format) instead.'
    
  # check if we have a specific string format
  if len(args) == 2 and isinstance(args[0], types.StringTypes) and isinstance(args[1], types.StringTypes):
    return _parse(klass, subklass, args[0], args[1])
    
  # check for empty parameters
  if len(args) == 0 and len(kargs) == 0:
    parts = time.localtime()
    if klass == datetime.datetime:
      return klass.__new__(subklass, parts[0], parts[1], parts[2], parts[3], parts[4], parts[5])
    elif klass == datetime.date:
      return klass.__new__(subklass, parts[0], parts[1], parts[2])
    raise SystemError, 'Invalid klass used in DateTime._create'
    
  # check for none type
  if len(args) >= 1 and args[0] == None:
    return None
    
  # revert to the datetime or date constructor
  return klass.__new__(subklass, *args, **kargs)
    

class Date(datetime.date):
  '''The Picalo date type to represent dates.'''
  
  def __new__(*args, **kargs):
    '''Date objects can be created in many different ways:
       
       Option 1: No arguments for now.
         - Date()
         
       Option 2: Provide the individual parts as numbers.  The year is
       the only one that is absolutely necessary.
         - Date(year, month, day, hour, minute, second)
         - Date(year, month, day)
         - Date(year)
  
       Option 3: Converts a string using a custom format.  Format options are
       described in the strftime manpage.  Search the web for
       "strftime unix manpage" for the options.
         - Date('2010-01-31', '%Y-%m-%d') 
         
       Option 4: Converts a string using ISO 8601 date format or a few other common formats
         - Date('2010-01-31')      
         - Date('10-01-31')      
         - Date('01-31-2010')      
         - Date('2010 01 31')      
         - Date('10 01 31')      
         - Date('01 31 2010')      
         - Date('2010.01.31')      
         - Date('10.01.31')      
         - Date('01.31.2010')      
         - Date('2010/01/31')      
         - Date('10/01/31')      
         - Date('01/31/2010')      
         - Date('2010-Jan-01')          
         - Date('25 JAN 2005')          
         - Date('25 Jan 2010')    
         - Date('25 January 2005')      
         - Date('2010-January-01')      
         - Date('January 1, 2010')      
         - Date('Jan 1, 2010')          
         
       Time can be specified with any of the above formats with hh:mm:ss, as in
         - Date('2010-01-31 14:15:25')
         - Date('2010-01-31 02:15:25am')
         - Date('2010-01-31 02:15:25 am')
       but with the Date() function, the time information is ignored.  Use
       DateTime if you want to keep time information as well.
    '''
    return _create(datetime.date, *args, **kargs)


  def __reduce_ex__(self, pickle_protocol):
    '''Helps in saving and loading the object'''
    return (Date, (self.strftime('%Y-%m-%d %H:%M:%S'), ))
    

  def __add__(self, other):   return datetime.date.__add__(self, __coerce_value__(other))
  def __radd__(self, other):  return datetime.date.__radd__(self, __coerce_value__(other))
  def __sub__(self, other):   return datetime.date.__sub__(self, __coerce_value__(other))
  def __rsub__(self, other):  return datetime.date.__rsub__(self, __coerce_value__(other))

    

class DateTime(datetime.datetime):
  '''The Picalo date/time type to represent dates and times.'''
  def __new__(*args, **kargs):
    '''The Picalo date/time type to represent dates and times.
       DateTime objects can be created in many different ways:
       
       Option 1: No arguments.
         - DateTime()
         
       Option 2: Provide the individual parts as numbers.  The year is
       the only one that is absolutely necessary.
         - DateTime(year, month, day, hour, minute, second)
         - DateTime(year, month, day)
         - DateTime(year)
  
       Option 3: Converts a string using a custom format.  Format options are
       described in the strftime manpage.  Search the web for
       "strftime unix manpage" for the options.
         - DateTime('2010-01-31', '%Y-%m-%d') 
         
       Option 4: Converts a string using ISO 8601 date format or a few other common formats
         - DateTime('2010-01-31')      
         - DateTime('10-01-31')      
         - DateTime('01-31-2010')      
         - DateTime('2010 01 31')      
         - DateTime('10 01 31')      
         - DateTime('01 31 2010')      
         - DateTime('2010.01.31')      
         - DateTime('10.01.31')      
         - DateTime('01.31.2010')      
         - DateTime('2010/01/31')      
         - DateTime('10/01/31')      
         - DateTime('01/31/2010')      
         - DateTime('2010-Jan-01')          
         - DateTime('25 JAN 2005')          
         - DateTime('25 Jan 2010')    
         - DateTime('25 January 2005')      
         - DateTime('2010-January-01')      
         - DateTime('January 1, 2010')      
         - DateTime('Jan 1, 2010')          
         
       Time can be specified with any of the above formats with hh:mm:ss, as in
         - DateTime('2010-01-31 14:15:25')
         - DateTime('2010-01-31 02:15:25am')
         - DateTime('2010-01-31 02:15:25 am')
         - Time zone information is not supported with Option 4 at this point.
    '''
    return _create(datetime.datetime, *args, **kargs)


  def __reduce_ex__(self, pickle_protocol):
    '''Helps in saving and loading the object'''
    return (DateTime, (self.strftime('%Y-%m-%d %H:%M:%S'), ))

  def __add__(self, other):   return datetime.datetime.__add__(self, __coerce_value__(other))
  def __radd__(self, other):  return datetime.datetime.__radd__(self, __coerce_value__(other))
  def __sub__(self, other):   return datetime.datetime.__sub__(self, __coerce_value__(other))
  def __rsub__(self, other):  return datetime.datetime.__rsub__(self, __coerce_value__(other))


def DateTimeFormat(datetime_value, format=None):
  '''Formats a DateTime or Date object for printing using the
     given format.  Search the web for "strftime unix manpage" for 
     the formatting tokens.
  '''
  assert isinstance(datetime_value, (datetime.datetime, datetime.date, types.NoneType)), 'The datetime_value must be a DateTime object: ' + str(datetime_value)
  assert isinstance(format, (types.StringType, types.UnicodeType, types.NoneType)), 'The format must be a string or None: ' + str(format)
  if datetime_value == None:
    return datetime_value
  if format:
    return datetime_value.strftime(format)
  if isinstance(datetime_value, datetime.datetime):  # override default formatting for datetime
    return datetime_value.strftime('%Y-%m-%d %H:%M:%S')
  return str(datetime_value)
  
# alias to the DateTimeFormat
DateFormat = DateTimeFormat


# function to create a TimeDelta
def TimeDelta(days=0, seconds=0, microseconds=0, milliseconds=0, minutes=0, hours=0, weeks=0):
  '''A duration in time, such as the difference between two DateTimes.  This is often
     used in Grouping.stratify_by_date and Grouping.summarize_by_date.  Negative numbers
     specify durations that go backwards in time.
     
     @param   days:         The number of days in this duration
     @type    days:         int, long, or float
     @param   seconds:      The number of seconds in this duration
     @type    seconds:      int, long, or float
     @param   microseconds: The number of microseconds in this duration
     @type    microseconds: int, long, or float
     @param   milliseconds: The number of milliseconds in this duration
     @type    milliseconds: int, long, or float
     @param   minutes:      The number of minutes in this duration
     @type    minutes:      int, long, or float
     @param   hours:        The number of hours in this duration
     @type    hours:        int, long, or float
     @param   weeks:        The number of weeks in this duration
     @type    weeks:        int, long, or float
     @return: A datetime.timedelta object
     @rtype:  datetime.timedelta
  '''
  return datetime.timedelta(days, seconds, microseconds, milliseconds, minutes, hours, weeks)
  
# alias to a TimeDelta
DateDelta = TimeDelta





#!/usr/bin/env python
# -*- coding: windows-1251 -*-

#  Copyright (C) 2005 Roman V. Kiseliov
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
# 
#  3. All advertising materials mentioning features or use of this
#     software must display the following acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  4. Redistributions of any form whatsoever must retain the following
#     acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
#  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
#  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
#  OF THE POSSIBILITY OF SUCH DAMAGE.


__rev_id__ = """$Id: Column.py,v 1.4 2005/07/20 07:24:11 rvk Exp $"""


from BIFFRecords import ColInfoRecord
from Deco import *
from Worksheet import Worksheet


class Column(object):
    @accepts(object, int, Worksheet)
    def __init__(self, indx, parent_sheet):
        self._index = indx
        self._parent = parent_sheet
        self._parent_wb = parent_sheet.get_parent()
        self._xf_index = 0x0F
        
        self.width = 0x0B92
        self.hidden = 0
        self.level = 0
        self.collapse = 0


    def get_biff_record(self):
        options =  (self.hidden & 0x01) << 0
        options |= (self.level & 0x07) << 8
        options |= (self.collapse & 0x01) << 12
        
        return ColInfoRecord(self._index, self._index, self.width, self._xf_index, options).get()
        
        
        



# This code from 

# PEP: 318 
# Title: Decorators for Functions and Methods 
# Version: 1.35 
# Last-Modified: 2004/09/14 07:34:23 
# Author: Kevin D. Smith, Jim Jewett, Skip Montanaro, Anthony Baxter 

# This code fixes error in example 4: 
# authors' code contains
#       @accepts(int, (int,float))
#       @returns((int,float))
#       def func(arg1, arg2):
#           return arg1 * arg2
# 
# It should be:
#       @returns((int,float))
#       @accepts(int, (int,float))
#       def func(arg1, arg2):
#           return arg1 * arg2
# because of decorators are applied from bottom to up.


__rev_id__ = """$Id: Deco.py,v 1.4 2005/07/20 07:24:11 rvk Exp $"""


def accepts(*types):
    #print types
    def check_accepts(f):
        assert len(types) == f.func_code.co_argcount
        def new_f(*args, **kwds):
            for (a, t) in zip(args, types):
                assert isinstance(a, t), \
                       "arg %r does not match %s" % (a,t)
            return f(*args, **kwds)
        new_f.func_name = f.func_name
        return new_f
    return check_accepts

def returns(rtype):
    def check_returns(f):
        def new_f(*args, **kwds):
            result = f(*args, **kwds)
            assert isinstance(result, rtype), \
                   "return value %r does not match %s" % (result,rtype)
            return result
        new_f.func_name = f.func_name
        return new_f
    return check_returns


if __name__ == '__main__':
    import types 

    @returns(types.NoneType)
    @accepts(int, (int,float))
    def func(arg1, arg2):
        #return str(arg1 * arg2)
        pass

    func(1, 2)      



#!/usr/bin/python

####################################################################################
#                                                                                  #
# Copyright (c) 2003 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#
# Version 1.00
# Jun 22, 2005: Added summary_analyze.
#               Made the analyze function ignore numbers that don't have enough digits
# Dec 10, 2003: Version 1.00.  Tested using unittest.  
# Dec 08, 2003: Started this log.
#
#
#
####################################################################################

__doc__ = '''
The Benfords module performs digital analyses on data sets.  In the 1930's, Benford
realized that many number sets (now known to include invoice amounts and stock prices)
followed a certain pattern.  A 1 appeared as the first digit about 30 percent of the time.
Each digit in the number has a probability associated with which number it might be.

Numbers that are created by people (who obviously don't know about Benford's Law)
do not follow Benford's distribution.  In recent years, Benford's Law has been used
to separate values that occur naturally in business and those that are fabricated.
'''

  
    

# a listing of the public functions in this module (used by Manual.py)
__functions__ = (
  'analyze',
  'calc_benford',
  'get_expected',
)



import math, types
from lib import stats
import Grouping, Simple
from picalo import Table, TableArray, check_valid_table, run_tablearray
from Number import number


def calc_benford(position, digit, base=10):
  '''Helper function that codes benford's actual formula
     The generalized formula was found at http://www.mathpages.com/home/kmath302/kmath302.htm
     This method calculates the probability at a given digit is in a given position.
     
     @param position:  The position in the number (0=first digit, 1=second digit, ...)
     @type  position:  int
     @param digit:     The actual number (0,1,2,3,4,5,6,7,8,9) this digit is
     @type  digit:     int
     @param base:      The number base.  Optional (default is 10)
     @type  base:      int
     @return:          The Benford probability (the percentage of the time this position will have this digit).
     @rtype:           float
  '''
  assert isinstance(position, (types.IntType, types.LongType, types.FloatType)), 'Invalid position: ' + str(position)
  assert isinstance(digit, (types.IntType)), 'Invalid digit: ' + str(digit)
  assert isinstance(base, (types.IntType)), 'Invalid base: ' + str(base)
  part1 = 1 / math.log(base)
  if position == 0: k = 0
  else: k = math.pow(base, position - 1)  # bottom of the summation
  k_stop = math.pow(base, position) - 1 # top of the summation
  part2 = 0.0
  while k <= k_stop:
    part2 += math.log(1.0 + (1.0 / (k * base + digit)))
    k += 1
  return part1 * part2
  

def get_expected(number, number_of_significant_digits=1):
  '''Calculates Benford's expected probability for a given number to a certain number of digits.
     For example, given the number 1234, calulate the combined probability that a 1 appears
     as the first digit, a 2 appears as the second digit, and so forth, to the number of desired
     significant digits.
     
     @param number:                       The number (1234 in the example) to calculate the probability for
     @type  number:                       float
     @param number_of_significant_digits: The number of positions to use for the probability.  In the example, a value of 2 means to calculate the probability for the number 12.
     @type  number_of_significant_digits: int
     @return:                             The expected frequency of this number according to Benford's Law.
     @rtype:                              float
  '''
  assert isinstance(number, (types.IntType, types.LongType, types.FloatType, types.StringType, types.UnicodeType)), 'Invalid number: ' + str(number)
  assert isinstance(number_of_significant_digits, types.IntType), 'Invalid number of significant digits: ' + str(number_of_significant_digits)
  probability = 1.0
  numst = str(number)
  assert len(numst) >= number_of_significant_digits, str(len(numst)) + ' is not long enough to calculate a Benford probability at ' + str(number_of_significant_digits) + ' digits.'
  for pos in range(0, number_of_significant_digits):
    probability *= calc_benford(pos, int(numst[pos]))  # multiply since it is a statistical AND
  return probability


    
class Result:
  '''A simple class to represent a result statistic for an individual data point'''
  def __init__(self, number, significant_digits):
    self.number = number
    self.significant_digits = significant_digits
    self.frequency = 0
    self.expected = 0
    
  def __repr__(self):
    return 'Result (num, sign, freq, exp, diff): ' + '\t'.join([ str(self.number), self.significant_digits, str(self.frequency), str(self.expected), str(math.fabs(self.frequency - self.expected)) ])



def analyze(table, col, number_of_significant_digits=1):
  '''Performs a Benford's analysis on a table column.  Returns a picalo table.
     The frequency and expected values will be 0 for items that were not analyzed 
     (due to insufficient digits).
     
     Important: if you ask for 2 significant digits, any input numbers that
     do not have two digits are ignored and not included in the results.  If these 
     numbers were not ignored, the analysis would throw errors.
     
     Example:
     
      >>> table = Table([('col000', unicode), ('col001', int), ('col002', int)], [
      ...             ['Dan',10,8],
      ...             ['Sally',12,12],
      ...             ['Dan',11,15], 
      ...             ['Sally',12,14], 
      ...             ['Dan',11,16], 
      ...             ['Sally',15,15], 
      ...             ['Dan',16,15], 
      ...             ['Sally',13,14]])
      >>> results = Benfords.analyze(table, 1, number_of_significant_digits=2)            
      >>> results.view()
      +--------+--------------------+------------------+----------------------+-----------------+
      | Number | Significant Digits | Actual Frequency | Expected Probability |    Difference   |
      +--------+--------------------+------------------+----------------------+-----------------+
      |     10 | 10                 |            0.125 |      0.0360270497068 | 0.0889729502932 |
      |     12 | 12                 |             0.25 |      0.0327585353738 |  0.217241464626 |
      |     11 | 11                 |             0.25 |      0.0342843373349 |  0.215715662665 |
      |     12 | 12                 |             0.25 |      0.0327585353738 |  0.217241464626 |
      |     11 | 11                 |             0.25 |      0.0342843373349 |  0.215715662665 |
      |     15 | 15                 |            0.125 |      0.0291027478744 | 0.0958972521256 |
      |     16 | 16                 |            0.125 |      0.0281085963079 | 0.0968914036921 |
      |     13 | 13                 |            0.125 |       0.031406327064 |  0.093593672936 |
      +--------+--------------------+------------------+----------------------+-----------------+     
     
     Example 2:
     I usually add a column for the difference from Benford's expectation to the table, 
     then summarize to get an average difference per vendor, employee, etc.
     Individual numbers will often not match Benford, but averages across several 
     numbers should match.

      >>> table = Table([('col000', unicode), ('col001', int), ('col002', int)], [
      ...             ['Dan',10,8],
      ...             ['Sally',12,12],
      ...             ['Dan',11,15], 
      ...             ['Sally',12,14], 
      ...             ['Dan',11,16], 
      ...             ['Sally',15,15], 
      ...             ['Dan',16,15], 
      ...             ['Sally',13,14]])
      >>> table.append_column('ben_diff', Benfords.analyze(table, 1, 2).column(4))
      >>> table.view()
      +--------+--------+--------+-----------------+
      | col000 | col001 | col002 |     ben_diff    |
      +--------+--------+--------+-----------------+
      | Dan    |     10 |      8 | 0.0889729502932 |
      | Sally  |     12 |     12 |  0.217241464626 |
      | Dan    |     11 |     15 |  0.215715662665 |
      | Sally  |     12 |     14 |  0.217241464626 |
      | Dan    |     11 |     16 |  0.215715662665 |
      | Sally  |     15 |     15 | 0.0958972521256 |
      | Dan    |     16 |     15 | 0.0968914036921 |
      | Sally  |     13 |     14 |  0.093593672936 |
      +--------+--------+--------+-----------------+
      >>> results = Grouping.summarize_by_value(table, 'col000', 
      ...           ben_avg="sum(group['ben_diff']) / len(group)")
      >>> results.view()
      +----------+--------+----------------+
      | StartKey | EndKey |    ben_avg     |
      +----------+--------+----------------+
      | Dan      | Dan    | 0.154323919829 |
      | Sally    | Sally  | 0.155993463579 |
      +----------+--------+----------------+

     @param table:                         A Picalo table
     @type  table:                         Table
     @param col:                           The column to be analyzed.
     @type  col:                           str
     @param number_of_significant_digits:  The number of leading digits to use in the analysis.  Higher numbers (3-5) require more data for statistical power.
     @type  number_of_significant_digits:  int
     @return:                              A Picalo table describing the results of the analysis.
     @rtype:                               Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Calculating digital analysis...', analyze, table, col, number_of_significant_digits=number_of_significant_digits)
  check_valid_table(table, col)
  assert isinstance(number_of_significant_digits, types.IntType), 'Invalid number of significant digits: ' + str(number_of_significant_digits)
  list_of_numbers = table.column(col)
  # calculate the distribution of the digits
  digits = {}
  stats = Table([ ('Number', int), ('SignificantDigits', int), ('ActualFrequency', float), ('ExpectedProbability', float)])
  stats.append_calculated('Difference', number, "abs(record['ActualFrequency'] - record['ExpectedProbability'])")
  for num in list_of_numbers:
    numst = str(num)[0: number_of_significant_digits]
    if len(numst) != number_of_significant_digits:  # ignore any numbers that don't have enough digits
      continue
    stats.append([num, numst, 0, 0])
    if len(numst) == number_of_significant_digits:
      if digits.has_key(numst): 
        count = digits[numst] + 1
      else: 
        count = 1
      digits[numst] = count  
  # add the frequency and expected to the stats
  total_count = float(len(list_of_numbers))
  for stat in stats:
    count = digits[str(stat['SignificantDigits'])]
    stat['ActualFrequency'] = count / total_count
    try:
      stat['ExpectedProbability'] = get_expected(stat['SignificantDigits'], number_of_significant_digits) 
    except:
      stat['ActualFrequency'] = 0
      stat['ExpectedProbability'] = 0
    
  # return the statistics
  return stats






####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################

import sys
  
# global functions (defined in this file)
__all__ = [
  'error',
]

####################################
###   The error type for picalo
###   When errors are thrown in calculations,
###   an error object is placed in the cell.
###   The error shows in the cell as <err>,
###   but as an object allows the user to
###   find out the specifics of the error.

class error:
  def __init__(self, exc=None, msg=None, previous=None):
    '''An error type that signifies that an error occurred when
       calculating the value for some cell.  Don't create this
       directly.  Picalo creates them when errors
       are encountered during analyses so the analysis doesn't 
       get stopped by small errors.
       
       The error object is useful in that it encodes information
       about what went wrong.  Although it only prints as <err>
       in table listings, it can be inspected as shown in the 
       set_type method example.
       
       @param exc:         The exception object.
       @type  exc:         Exception
       @param msg:         Allows you to provide a specific error message.  Defaults to str(exception).
       @type  msg:         str
       @param previous:    The previous value of the cell, so we don't lose it.
       @type  previous:    object
    '''
    self.msg = msg
    self.exc = None
    self.previous = previous
    if exc:
      if msg == None:  # default the message if not explicitly set
        self.msg = str(exc)
      if isinstance(exc, Exception):
        self.exc = exc
    if self.msg == None:  # ensure we have at least some message
       self.msg = 'Unspecified Error'
    
  def __repr__(self):
    return '<err: ' + str(self.msg) + '>'
    
  def __str__(self):
    return '<err: ' + str(self.msg) + '>'
    
  def get_message(self):
    '''Returns the actual error message.  This allows the object
       to be interrogated for what really went wrong.
     
       @return:  The error message.
       @rtype:   str
    '''
    return self.msg
    
  def get_exception(self):
    '''Returns the exception object associated with this error.
       This might be None if no object was created.
       
       @return:   The exception object that caused this error.
       @rtype:    Exception
    '''
    return self.exc
    
  def get_previous(self):
    '''Returns the previous value of the cell before the error
       was placed in the table.  This might be none if the 
       error didn't replace a cell value.
       
       @return:   The previous value of the cell where this error occurred.
       @rtype:    object
    '''
    return self.previous
    
  # the error object replaces other variables in table cells, and
  # therefore is sent into many routines.  therefore, it must be
  # able to act like the object it replaced (str, int, etc.) or
  # it will cause additional errors
  def __lt__(self, other): return id(self) < id(other)   # compare ids to get consistency for the algorithm
  def __le__(self, other): return id(self) <= id(other)
  def __eq__(self, other): return id(self) == id(other)
  def __ne__(self, other): return id(self) != id(other)
  def __gt__(self, other): return id(self) > id(other)
  def __ge__(self, other): return id(self) >= id(other)
  def __cmp__(self, *args, **kargs): return cmp(id(self, *args, **kargs), id(other))
  def __nonzero__(self, *args, **kargs): return True
  def __len__(self, *args, **kargs): return 0
  def __getitem__(self, key): return self
  def __setitem__(self, key, value): pass
  def __delitem__(self, key): pass
  def __iter__(self, *args, **kargs): return iter([])
  def __contains__(self, item): return False
  def __add__(self, *args, **kargs): return self
  def __sub__(self, *args, **kargs): return self
  def __mul__(self, *args, **kargs): return self
  def __floordiv__(self, *args, **kargs): return self
  def __mod__(self, *args, **kargs): return self
  def __divmod__(self, *args, **kargs): return self
  def __pow__(self, *args, **kargs): return self
  def __lshift__(self, *args, **kargs): return self
  def __rshift__(self, *args, **kargs): return self
  def __and__(self, *args, **kargs): return self
  def __xor__(self, *args, **kargs): return self
  def __or__(self, *args, **kargs): return self
  def __div__(self, *args, **kargs): return self
  def __truediv__(self, *args, **kargs): return self
  def __radd__(self, *args, **kargs): return self
  def __rsub__(self, *args, **kargs): return self
  def __rmul__(self, *args, **kargs): return self
  def __rdiv__(self, *args, **kargs): return self
  def __rtruediv__(self, *args, **kargs): return self
  def __rfloordiv__(self, *args, **kargs): return self
  def __rmod__(self, *args, **kargs): return self
  def __rdivmod__(self, *args, **kargs): return self
  def __rpow__(self, *args, **kargs): return self
  def __rlshift__(self, *args, **kargs): return self
  def __rrshift__(self, *args, **kargs): return self
  def __rand__(self, *args, **kargs): return self
  def __rxor__(self, *args, **kargs): return self
  def __ror__(self, *args, **kargs): return self
  def __iadd__(self, *args, **kargs): return self
  def __isub__(self, *args, **kargs): return self
  def __imul__(self, *args, **kargs): return self
  def __idiv__(self, *args, **kargs): return self
  def __itruediv__(self, *args, **kargs): return self
  def __ifloordiv__(self, *args, **kargs): return self
  def __imod__(self, *args, **kargs): return self
  def __ipow__(self, *args, **kargs): return self
  def __ilshift__(self, *args, **kargs): return self
  def __irshift__(self, *args, **kargs): return self
  def __iand__(self, *args, **kargs): return self
  def __ixor__(self, *args, **kargs): return self
  def __ior__(self, *args, **kargs): return self
  def __neg__(self, *args, **kargs): return self
  def __pos__(self, *args, **kargs): return self
  def __abs__(self, *args, **kargs): return self
  def __invert__(self, *args, **kargs): return self
  def __complex__(self, *args, **kargs): return complex(0)
  def __int__(self, *args, **kargs): return 0
  def __long__(self, *args, **kargs): return 0
  def __float__(self, *args, **kargs): return 0.0
  def __oct__(self, *args, **kargs): return str(self, *args, **kargs)
  def __hex__(self, *args, **kargs): return str(self, *args, **kargs)
  def __coerce(self, other): return (self, other)



#!/usr/bin/env python
# -*- coding: windows-1251 -*-

#  Copyright (C) 2005 Roman V. Kiseliov
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
# 
#  3. All advertising materials mentioning features or use of this
#     software must display the following acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  4. Redistributions of any form whatsoever must retain the following
#     acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
#  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
#  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
#  OF THE POSSIBILITY OF SUCH DAMAGE.


__rev_id__ = """$Id: Cell.py,v 1.3 2005/08/11 08:53:48 rvk Exp $"""


import struct
import BIFFRecords


class StrCell(object):
    __slots__ = ["__init__", "get_biff_data",
                "__parent", "__idx", "__xf_idx", "__sst_idx"]

    def __init__(self, parent, idx, xf_idx, sst_idx):
        self.__parent = parent
        self.__idx = idx
        self.__xf_idx = xf_idx
        self.__sst_idx = sst_idx

    def get_biff_data(self):
        return BIFFRecords.LabelSSTRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, self.__sst_idx).get()


class BlankCell(object):
    __slots__ = ["__init__", "get_biff_data",
                "__parent", "__idx", "__xf_idx"]

    def __init__(self, parent, idx, xf_idx):
        self.__parent = parent
        self.__idx = idx
        self.__xf_idx = xf_idx

    def get_biff_data(self):
        return BIFFRecords.BlankRecord(self.__parent.get_index(), self.__idx, self.__xf_idx).get()


class MulBlankCell(object):
    __slots__ = ["__init__", "get_biff_data",
                "__parent", "__col1", "__col2", "__xf_idx"]

    def __init__(self, parent, col1, col2, xf_idx):
        self.__parent = parent
        self.__col1 = col1
        self.__col2 = col2
        self.__xf_idx = xf_idx

    def get_biff_data(self):
        return BIFFRecords.MulBlankRecord(self.__parent.get_index(), self.__col1, self.__col2, self.__xf_idx).get()


class NumberCell(object):
    __slots__ = ["__init__", "get_biff_data",
                "__parent", "__idx", "__xf_idx", "__number"]


    def __init__(self, parent, idx, xf_idx, number):
        self.__parent = parent
        self.__idx = idx
        self.__xf_idx = xf_idx
        self.__number = float(number)


    def get_biff_data(self):
        rk_encoded = 0

        packed = struct.pack('<d', self.__number)

        #print self.__number
        w0, w1, w2, w3 = struct.unpack('<4H', packed)
        if w0 == 0 and w1 == 0 and w2 & 0xFFFC == w2:
            # 34 lsb are 0
            #print "float RK"
            rk_encoded = (w3 << 16) | w2
            return BIFFRecords.RKRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, rk_encoded).get()

        if abs(self.__number) < 0x40000000 and int(self.__number) == self.__number:
            #print "30-bit integer RK"
            rk_encoded = 2 | (int(self.__number) << 2)
            return BIFFRecords.RKRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, rk_encoded).get()

        temp = self.__number*100
        packed100 = struct.pack('<d', temp)
        w0, w1, w2, w3 = struct.unpack('<4H', packed100)
        if w0 == 0 and w1 == 0 and w2 & 0xFFFC == w2:
            # 34 lsb are 0
            #print "float RK*100"
            rk_encoded = 1 | (w3 << 16) | w2
            return BIFFRecords.RKRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, rk_encoded).get()

        if abs(temp) < 0x40000000 and int(temp) == temp:
            #print "30-bit integer RK*100"
            rk_encoded = 3 | (int(temp) << 2)
            return BIFFRecords.RKRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, rk_encoded).get()

        #print "Number" 
        #print
        return BIFFRecords.NumberRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, self.__number).get()


class MulNumberCell(object):
    __slots__ = ["__init__", "get_biff_data"]

    def __init__(self, parent, idx, xf_idx, sst_idx):
        self.__parent = parent
        self.__idx = idx
        self.__xf_idx = xf_idx
        self.__sst_idx = sst_idx


    def get_biff_data(self):
        raise Exception


class FormulaCell(object):
    __slots__ = ["__init__", "get_biff_data",
                "__parent", "__idx", "__xf_idx", "__frmla"]

    def __init__(self, parent, idx, xf_idx, frmla):
        self.__parent = parent
        self.__idx = idx
        self.__xf_idx = xf_idx
        self.__frmla = frmla


    def get_biff_data(self):
        return BIFFRecords.FormulaRecord(self.__parent.get_index(), self.__idx, self.__xf_idx, self.__frmla.rpn()).get()






#!/usr/bin/python

####################################################################################
#                                                                                  #
# Copyright (c) 2003 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#
# Version 1.00
# Jun 20, 2005: Changed "Group by" to "Stratify by" throughout.
# May 26, 2005: Changed all the lambda stuff over to expressions that go through eval
# Jan 14, 2003: Fixed minor errors in the documentation for summary methods
# Dec 14, 2003: Version 1.00.  Refactored the code.  Included the functions
#               from the previous Summarize module.
# Dec 08, 2003: Version 0.30.  Started this log.
#                  
#
#
####################################################################################

__doc__ = '''
The Grouping module contains functions that stratify and summarize records in different ways.
Grouping is a basic fraud detection method that helps generate norms and compare
records against those norms.  Further, it splits data sets into individual groups
that should be analyzed separately.

The Grouping module only stratifiess a table into many tables.  It doesn't do any summarization
of values.  All detail data are still in the tables.

The summary routines not only stratifies data on key, but also summarizes the detail
tables using sum, mean, and other statistical routines.  You get only one table returned
from Summarize.  Stratifying gives the intermediate result -- the full detail in many tables. 

For those familiar with SQL, the summarize routines are similar to the GROUP BY keyword.  Groups
are collapsed into summary records and a single table is returned.  The stratifying functions are 
similar to running one query to retrieve all unique key values, followed by a query to retrieve 
the records that match each key value.  The result is many tables (one per unique key value).
'''



# a listing of the public functions in this module (used by Manual.py)
__functions__ = [
  'stratify',
  'stratify_by_value',
  'stratify_by_expression',
  'stratify_by_step',
  'stratify_by_date',
  'summarize',
  'summarize_by_expression',
  'summarize_by_value',
  'summarize_by_step',
  'summarize_by_date',
]



import Simple
import math, sys, types, datetime
from picalo import Table, TableArray, show_progress, clear_progress, check_valid_table
from picalo import run_tablearray, ensure_valid_variables, make_unique_colnames, ensure_unique_colname
from Expression import PicaloExpression
from Calendar import DateDelta, Date, DateTime


def stratify(table, number_of_groups):
  '''Stratifies a Picalo table into a specified number of sub-tables.
     The table should be sorted appropriately before this function is called.
     This function does not modify the underlying table.
     
     The start and end record indices are recorded in table.startvalue
     and table.endvalue.  See the example for more information.
     
     Example:
       >>> table = Table([('col000', int), ('col001', int)], [[1,1],[2,2],[3,3]])
       >>> groups = Grouping.stratify(table, 2)
       >>> print groups[0].startvalue, groups[0].endvalue
       0, 1
       >>> groups[0].view()
       +--------+--------+
       | col000 | col001 |
       +--------+--------+
       |      1 |      1 |
       |      2 |      2 |
       +--------+--------+
       >>> print groups[1].startvalue, groups[1].endvalue
       3, 3
       >>> groups[1].view()
       +--------+--------+
       | col000 | col001 |
       +--------+--------+
       |      3 |      3 |
       +--------+--------+
     
     @param table:            The table to be stratified
     @type  table:            Table
     @param number_of_groups: The number of sub-tables to create
     @type  number_of_groups: int
     @return:                 A list of new tables
     @rtype:                  TableArray
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Stratifying...', stratify, table, number_of_groups)
  check_valid_table(table)
  assert isinstance(number_of_groups, types.IntType), 'The number of groups must be an integer.'
  try:
    # make the groups
    groups = TableArray()
    span = int(math.ceil( float(len(table)) / number_of_groups ))
    for i in range(0, len(table), span):
      show_progress('Stratifying...', float(i) / len(table))
      g = table[i: i+span]
      g.startvalue = i
      g.endvalue = i+span-1
      groups.append(g)
    return groups
  finally:
    clear_progress()
  

def stratify_by_value(table, *cols):
  '''Stratifies a Picalo table by composite key (combination of values in the cols columns).
     A new table is created for each unique composite key, resulting in a list of tables.
     This function does not modify the underlying table.
     
     Each key is recorded as the start and end values of each group.
     See the example for more information.
     
     Example:
        >>> table = Table([('col000', int), ('col001', int)], [['Dan',10],['Sally',10],['Dan',11]])
        >>> groups = Grouping.stratify_by_value(table, 'col000')
        >>> groups[0].view()
        +--------+--------+
        | col000 | col001 |
        +--------+--------+
        | Dan    |     10 |
        | Dan    |     11 |
        +--------+--------+
        >>> groups[1].view()
        +--------+--------+
        | col000 | col001 |
        +--------+--------+
        | Sally  |     10 |
        +--------+--------+
     
     @param table:  The table to be stratified
     @type  table:  Table
     @param cols:   The remaining parameters are the column names/indices to stratify by
     @type  cols:   str
     @return:       A list of new tables, each table containing rows with the same key values.
     @rtype:        TableArray
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Stratifying...', stratify_by_value, table, *cols)
  check_valid_table(table, *cols)
  try:
    index = table.index(*cols)
    # put the records into the groups in the index
    groups = TableArray()
    keys = index.keys()
    keys.sort()
    for i, key in enumerate(keys):
      show_progress('Stratifying...', float(i) / len(keys))
      group = Table(table.columns, [ table[i] for i in index[key] ])
      group.startvalue = group.endvalue = key
      groups.append(group)
    return groups  
  finally:
    clear_progress()
  
  
def stratify_by_expression(table, expression):
  '''Stratifies a table based upon the return value from an expression.
     For each record in the table, the expression is evaluated with the following variables:
       1. startrecord => the starting record of the current group.
       2. record => the current record being evaluated.
     If the expression evaluates to True, rec is placed in a new group and becomes startrec
     If the expression evaluates to False, rec is placed in the current group.

     The start and end record indices are recorded in table.startvalue
     and table.endvalue.  See the example for more information.
     
     Example (starts a new group on each odd value in column 1):
       >>> table1 = Table([('col000', int)], [[1],[2],[3],[4]])
       >>> groups = Grouping.stratify_by_expression(table1, "record[0] % 2.0 == 1.0")
       >>> print groups[0].startvalue, groups[0].endvalue
       0, 1
       >>> groups[0].view()
       +--------+
       | col000 |
       +--------+
       |      1 |
       |      2 |
       +--------+
       >>> print groups[1].startvalue, groups[1].endvalue
       3, 4
       >>> groups[1].view()
       +--------+
       | col000 |
       +--------+
       |      3 |
       |      4 |
       +--------+     
     
     @param table:       The table to be stratified
     @type  table:       Table
     @param expression:  An expression that evaluates the current record and returns whether a new group should be started
     @type  expression:  str
     @return:            A list of new tables, each containing rows from table that were grouped together
     @rtype:             TableArray
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Stratifying...', stratify_by_expression, table, expression)
  check_valid_table(table)
  assert isinstance(expression, (types.StringType, types.UnicodeType, PicaloExpression)), 'Invalid expression: ' + str(expression)
  try:
    groups = TableArray()
    startrec = None 
    pe = PicaloExpression(expression)
    for i in range(len(table)):
      show_progress('Stratifying...', float(i) / len(table))
      currentrec = table[i]
      if startrec == None or pe.evaluate([{'startrecord':startrec, 'record':currentrec, 'recordindex':i}, currentrec, startrec]):
        currentgroup = Table(table.columns)
        currentgroup.startvalue = i
        groups.append(currentgroup)
        startrec = currentrec
      currentgroup.endvalue = i
      currentgroup.append(currentrec)
    return groups
  finally:
    clear_progress()
  

def stratify_by_step(table, col, step):
  '''Stratifies a table based upon the value of col.  Each time
     the value of col jumps > step, a new group is started.
  
     The table should be sorted correctly *before* this method is called.
     This method simply runs through the table records sequentially.
     
     The start and end values of each group are recorded in the table object
     since they may be different than the actual start and end column values.
     See the example for more information.
     
     Records are stratified where startvalue >= record[col] < endvalue.
  
     Example:    
       >>> table1 = Table([('col000', int), ('col001', int)], [[1,1], [2,2], [5.9,3], [6,1], [8,2], [16,1]])
       >>> groups = Grouping.stratify_by_step(table1, 0, 5)
       >>> print groups[0].startvalue, groups[0].endvalue
       1, 6
       >>> groups[0].view()
       +--------+--------+
       | col000 | col001 |
       +--------+--------+
       |      1 |      1 |
       |      2 |      2 |
       |    5.9 |      3 |
       +--------+--------+
       >>> print groups[1].startvalue, groups[1].endvalue
       6, 11
       >>> groups[1].view()
       +--------+--------+
       | col000 | col001 |
       +--------+--------+
       |      6 |      1 |
       |      8 |      2 |
       +--------+--------+
       >>> print groups[2].startvalue, groups[2].endvalue
       11, 16
       >>> groups[2].view()
       +--------+--------+
       | col000 | col001 |
       +--------+--------+
       +--------+--------+
       >>> print groups[3].startvalue, groups[3].endvalue
       16, 21
       >>> groups[3].view()
       +--------+--------+
       | col000 | col001 |
       +--------+--------+
       |     16 |      1 |
       +--------+--------+   
       
     @param table:  The table to be stratified
     @type  table:  Table
     @param col:    The column to use to step by
     @type  col:    str
     @param step:   The step value that starts a new group
     @type  step:   int
     @return:       A list of new tables, each containing rows from table that were stratified together
     @rtype:        TableArray
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Stratifying...', stratify_by_step, table, col, step)
  check_valid_table(table, col)
  if len(table) == 0:
    return TableArray([table])
  if isinstance(step, datetime.timedelta):
    assert table.column(col).get_type() in (Date, DateTime), 'The column to group by must be DateTime or Date type.'
    assert isinstance(step, datetime.timedelta), 'The step must be a DateDelta or TimeDelta type.'
  else:
    assert table.column(col).get_type() in (types.IntType, types.FloatType, types.LongType), 'The column to group by must be a number type.'
    assert isinstance(step, (types.IntType, types.FloatType, types.LongType)), 'The step must be number type such as int, long, or float.'
  try:
    Simple.sort(table, True, col)
    groups = TableArray()
    endval = None
    currentgroup = None
    for i, rec in enumerate(table):
      show_progress('Stratifying...', float(i) / len(table))
      while endval == None or rec[col] >= endval:
        currentgroup = Table(table.columns)
        groups.append(currentgroup)
        if endval == None: 
          currentgroup.startvalue = rec[col]
          endval = rec[col] + step
        else: 
          currentgroup.startvalue = endval
          endval += step
        currentgroup.endvalue = endval
      currentgroup.append(rec)
    return groups  
  finally:
    clear_progress()
    
  
    
def stratify_by_date(table, col, duration):
  '''Stratifies the table rows into specific groups by date ranges.  This function
     is useful to split a table into ranges such as two-week periods (useful for 
     analyzing timecard and invoice databases).
     
     Aging can also be done by sorting the table in reverse (newest to oldest dates)
     and using a negative duration.
     
     The start and end values of each group are recorded in the table object
     since they may be different than the actual start and end column values.
     See the example for more information.

     See the base.Calendar module for more information about time durations.
     
     Example:
       >>> table = Table([('col000', DateTime)], [
       ...   [DateTime(2000,1,1)],
       ...   [DateTime(2000,1,13)],
       ...   [DateTime(2000,1,14)],
       ...   [DateTime(2000,1,15)]
       ... ])
       >>> groups = Grouping.stratify_by_date(table, 0, DateDelta(7))
       >>> print groups[0].startvalue, groups[0].endvalue
       2000-01-01 00:00:00, 2000-01-08 00:00:00
       >>> groups[0].view()
       +------------------------+
       |         col000         |
       +------------------------+
       | 2000-01-01 00:00:00.00 |
       +------------------------+
       >>> print groups[1].startvalue, groups[1].endvalue
       2000-01-08 00:00:00, 2000-01-15 00:00:00
       >>> groups[1].view()
       +------------------------+
       |         col000         |
       +------------------------+
       | 2000-01-13 00:00:00.00 |
       | 2000-01-14 00:00:00.00 |
       +------------------------+
       >>> print groups[2].startvalue, groups[2].endvalue
       2000-01-15 00:00:00, 2000-01-22 00:00:00
       >>> groups[2].view()
       +------------------------+
       |         col000         |
       +------------------------+
       | 2000-01-15 00:00:00.00 |
       +------------------------+
     
  
     @param table:     The table to be stratified
     @type  table:     Table
     @param col:       The column to stratify by.  
     @type  col:       str
     @param duration:  The number of days or seconds to put into each group.
     @type  duration:  int
     @return:          A list of new tables, each containing rows from table that were stratified together
     @rtype:            TableArray
  '''
  # this really isn't any different than stratify by step, just the data types
  if not isinstance(duration, datetime.timedelta):
    duration = DateDelta(duration)
  return stratify_by_step(table, col, duration)
  
  
  

###############################################
###   Summarize functions

def summarize(groups, **expressions):
  '''Summarizes a sequence of groups by evaluating a series of expressions on each group.
     The result is a single table with one row representing each group in groups.
     This is analogous to the SQL GROUP BY command.

     Each item in expressions should be an expression that summarizes a group.
     It must evaluate to a single value summarizing the entire table.
     Most expressions will probably only summarize a single column as is done in the
     example.  Use the Table['colname'] method do get the desired column.
     
     Example:
       >>> from picalo.lib import stats
       >>> table = Table([('col000', int), ('col001', int), ('col002', int')], [[1,1,1],[1,1,2],[2,1,2],[2,1,3]])
       >>> groups = Grouping.stratify_by_value(table, 0, 1)
       >>> summary = Grouping.summarize(groups, sum="sum(group['col002'])", avg="stats.mean(group['col002'])")
       >>> summary.view()
       +------------+----------+-----+-----+
       | StartValue | EndValue | sum | avg |
       +------------+----------+-----+-----+
       |     (1, 1) |   (1, 1) |   3 | 1.5 |
       |     (2, 1) |   (2, 1) |   5 | 2.5 |
       +------------+----------+-----+-----+
     
     @param groups:         A TableArray of groups, probably created by one of the stratify_by_... routines.
     @type  groups:         TableArray
     @param expressions:    One or more colname=expression pairs to summarize by.
     @return:               A single table containing the summaries of the groups.
     @rtype:                Table
  '''
  assert isinstance(groups, TableArray), 'The groups variable must be a TableArray.'
  # if all members are TableArrays, we need to recurse
  for sub in groups:
    if not isinstance(sub, TableArray):
      break
  else:
    return run_tablearray('Summarizing...', summarize, groups, **expressions)
  # check the column names and expressions
  for colname, expression in expressions.items():
    assert isinstance(colname, types.StringTypes), 'Invalid column name given to an expression: ' + str(colname)
    assert isinstance(expression, (types.StringType, types.UnicodeType, PicaloExpression)), 'Invalid expression: ' + str(expression)
  # run the summary
  try:
    headings = make_unique_colnames(ensure_valid_variables([ 'StartValue', 'EndValue' ] + expressions.keys()))
    coltypes = [ unicode, unicode ] + [ float for i in headings ]
    results = Table(zip(headings, coltypes))
    for expname, exp in expressions.items():  # convert to picalo expressions for speed
      expressions[expname] = PicaloExpression(exp)
    for i in range(len(groups)):
      show_progress('Summarizing...', float(i) / len(groups))
      group = groups[i]
      rec = results.append()
      if hasattr(group, 'startvalue'):
        rec['StartValue'] = group.startvalue
      if hasattr(group, 'endvalue'):
        rec['EndValue'] = group.endvalue
      for expname, exp in expressions.items():
        rec[expname] = exp.evaluate([{'group':group}])
    return results
  finally:
    clear_progress()
  
  
  
########################################################################
###  The remaining summarize_by_* are just convenience functions that 
###  first call the stratify_by_... function and then summarize the groups.
###  These functions are provided mostly to included summarization in
###  the standard documentation and to ease summarization for users.
  
def summarize_by_value(table, *cols, **expressions):
  '''Stratifies a Picalo table by composite key (combination of values in the col_list columns).
     A new table is created for each unique composite key, resulting in a list of tables.
     The function then summarizes the list of groups by running a series of expressions on
     each group.  The result is single table with one row representing each group.
     This is analogous to the SQL GROUP BY command.  This function does not modify the 
     underlying table.
     
     Each item in expressions should be an expression that summarizes a group.
     It must evaluate to a single value summarizing the entire table.
     Most expressions will probably only summarize a single column as is done in the
     example.  Use the Table['colname'] method do get the desired column.
     
     Example:
      >>> from picalo import *
      >>> from picalo.lib import stats
      >>> table = Table([('col000', unicode), ('col001', int), ('col002', int)], [
      ...      ['Dan',10,8],
      ...      ['Sally',12,12],
      ...      ['Dan',11,15], 
      ...      ['Sally',12,14], 
      ...      ['Dan',11,16], 
      ...      ['Sally',15,15], 
      ...      ['Dan',16,15], 
      ...      ['Sally',13,14]])
      >>> results = Grouping.summarize_by_value(table, 'col000', 
      ...      sum="sum(group['col001'])", 
      ...      correlation="stats.spearmanr(list(group['col001']), list(['col002']))")
      >>> results.view()
      +-------+-----+--------------------------------------------+
      | Value | sum |                correlation                 |
      +-------+-----+--------------------------------------------+
      | Dan   |  48 | (0.55000000000000004, 0.45000000000651308) |
      | Sally |  52 |  (0.84999999999999998, 0.1499999999962324) |
      +-------+-----+--------------------------------------------+
      >>> # the first number in the correlation is the statistic, second number is the p-value

     
     @param table:          The table to be summarized
     @type  table:          Table
     @param cols:           One or more columns to summarize with the given expressions.
     @param expressions:    One or more colname=expression pairs to summarize by.
     @return:               A single table containing the summaries of the groups.
     @rtype:                Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Summarizing...', summarize_by_value, table, *cols, **expressions)
  check_valid_table(table, *cols)
  summary = summarize(stratify_by_value(table, *cols), **expressions)
  try:
    show_progress('Formatting...', 0 / 2)
    summary.delete_column('EndValue')
    show_progress('Formatting...', 1 / 2)
    summary.column('StartValue').set_name(ensure_unique_colname(summary, '_'.join([ table.column(col).name for col in cols ])))
  finally:
    clear_progress()
  return summary

  
def summarize_by_expression(table, stratifying_expression, **expressions):
  '''Summarizess a table based upon the return value from expressions.
     The function then summarizes the list of groups by running a series of functions on
     each group.  The result is single table with one row representing each group.
     This is analogous to the SQL GROUP BY command.  This function does not modify the 
     underlying table.
     
     Each item in expressions should be an expression that summarizes a group.
     It must evaluate to a single value summarizing the entire table.
     Most expressions will probably only summarize a single column as is done in the
     example.  Use the Table['colname'] method do get the desired column.

     @param table:                   The table to be summarized
     @type  table:                   Table
     @param stratifying_expression:  An expression that evaluates the current record and returns whether a new group should be started
     @type  stratifying_expression:  str
     @param expressions:             One or more colname=expression pairs to summarize by.
     @return:                        A single table containing the summaries of the groups.
     @rtype:                         Table
  '''
  return summarize(stratify_by_expression(table, stratifying_expression), **expressions)

def summarize_by_step(table, col, step, **expressions):
  '''Summarizes a table based upon the value of col.  Each time
     the value of col jumps > step, a new group is started.
     The function then summarizes the list of groups by running a series of functions on
     each group.  The result is single table with one row representing each group.
     This is analogous to the SQL GROUP BY command.  This function does not modify the 
     underlying table.
  
     Each item in expressions should be an expression that summarizes a group.
     It must evaluate to a single value summarizing the entire table.
     Most expressions will probably only summarize a single column as is done in the
     example.  Use the Table['colname'] method do get the desired column.
     
     Example:    
       >>> from picalo.lib import stats
       >>> table1 = Table([('col000', int), ('col001', int)], [[1,1], [2,2], [5.9,3], [6,1], [8,2], [16,1]])
       >>> summary = Grouping.summarize_by_step(table1, 0, 5, count="len(group)")
       >>> summary.view()
       +------------+----------+-------+
       | StartValue | EndValue | count |
       +------------+----------+-------+
       |          1 |        6 |     3 |
       |          6 |       11 |     2 |
       |         11 |       16 |     0 |
       |         16 |       21 |     1 |
       +------------+----------+-------+

     @param table:          The table to be summarized
     @type  table:          Table
     @param col:            The column to stratify by.
     @type  col:            str
     @param step:           The step value that starts a new group
     @type  step:           float
     @param expressions:    One or more colname=expression pairs to summarize by.
     @return:               A single table containing the summaries of the groups.
     @rtype:                Table
  '''
  return summarize(stratify_by_step(table, col, step), **expressions)
  
def summarize_by_date(table, col, num_days_in_groups, **expressions):
  '''Summarizes the table rows into specific groups by date ranges.  
     The function then summarizes the list of groups by running a series of functions on
     each group.  The result is single table with one row representing each group.
     This is analogous to the SQL GROUP BY command.  This function does not modify the 
     underlying table.
     
     Aging can also be done by sorting the table in reverse (newest to oldest dates)
     and using a negative num_days_in_groups.
     
     Each item in expressions should be an expression that summarizes a group.
     It must evaluate to a single value summarizing the entire table.
     Most expressions will probably only summarize a single column as is done in the
     example.  Use the Table['colname'] method do get the desired column.
     
     Example:
       >>> table = Table([('col000', DateTime), ('col001', int), ('col002', int)], [
       ...   [DateTime(2000,1,1)],
       ...   [DateTime(2000,1,13)],
       ...   [DateTime(2000,1,14)],
       ...   [DateTime(2000,1,15)]
       ... ])
       >>> summary = Grouping.summarize_by_date(table, 0, TimeDelta(7), first="group[0][0]", last="group[-1][0]")
       >>> summary.view()
       +---------------------+---------------------+---------------------+---------------------+
       |      StartValue     |       EndValue      |         last        |        first        |
       +---------------------+---------------------+---------------------+---------------------+
       | 2000-01-01 00:00:00 | 2000-01-08 00:00:00 | 2000-01-01 00:00:00 | 2000-01-01 00:00:00 |
       | 2000-01-08 00:00:00 | 2000-01-15 00:00:00 | 2000-01-14 00:00:00 | 2000-01-13 00:00:00 |
       | 2000-01-15 00:00:00 | 2000-01-22 00:00:00 | 2000-01-15 00:00:00 | 2000-01-15 00:00:00 |
       +---------------------+---------------------+---------------------+---------------------+

     @param table:              The table to be summarized
     @type  table:              Table
     @param col:                The column to stratify by.
     @type  col:                str
     @param num_days_in_groups: The number of days or seconds to put into each group.
     @type  num_days_in_groups: int
     @param expressions:        One or more colname=expression pairs to summarize by.
     @return:                   A single table containing the summaries of the groups.
     @rtype:                    Table
  '''
  return summarize(stratify_by_date(table, col, num_days_in_groups), **expressions)  



#!/usr/bin/env python
# -*- coding: windows-1251 -*-

#  Copyright (C) 2005 Roman V. Kiseliov
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
# 
#  3. All advertising materials mentioning features or use of this
#     software must display the following acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  4. Redistributions of any form whatsoever must retain the following
#     acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
#  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
#  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
#  OF THE POSSIBILITY OF SUCH DAMAGE.


__rev_id__ = """$Id: ExcelFormula.py,v 1.3 2005/08/11 08:53:48 rvk Exp $"""


import ExcelFormulaParser, ExcelFormulaLexer
import struct
from antlr import ANTLRException


class Formula(object):
    __slots__ = ["__init__", "text", "rpn", "__s", "__parser"]


    def __init__(self, s):
        try:
            self.__s = s
            lexer = ExcelFormulaLexer.Lexer(s)
            self.__parser = ExcelFormulaParser.Parser(lexer)
            self.__parser.formula()
        except ANTLRException:
            raise Exception, "can't parse formula " + s

    def text(self):
        return self.__s

    def rpn(self):
        '''
        Offset    Size    Contents
        0         2       Size of the following formula data (sz)
        2         sz      Formula data (RPN token array)
        [2+sz]    var.    (optional) Additional data for specific tokens

        '''
        return struct.pack("<H", len(self.__parser.rpn)) + self.__parser.rpn




#!/usr/bin/python

####################################################################################
#                                                                                  #
# Copyright (c) 2003 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################

import re, os, os.path, types, datetime, inspect, gzip, pickle
from picalo import Table, show_progress, clear_progress, check_valid_table, Date, DateTime, DateTimeFormat, number
import picalo

__doc__ = '''
  This module provides easy access to DB-API 2.0 tables.  It is a decorator
  for connections and cursors.  Cursor results can be accessed efficiently
  via field name or index.  
  
  Right now ODBC data sources are the primary mechanism to access databases.
  Set up ODBC in your operating system and call the OdbcConnection method
  to create a connection.  In addition, direct connections to MySQL and
  PostgreSQL are supported (without the need for an ODBC DSN setup).
  
  The primary use of this module is table(), which returns a standard
  Picalo table.  Other methods are provided for 
  advanced users who want more control over the databse connection.
  
  Most users only need to learn how to 1) create Connection objects,
  and 2) run conn.table() to query database tables.
       
  The module also provides classes to ease the creation of insert, update, and
  select queries.  
'''
  
__functions__ = (
  'SqliteConnection',
  'OdbcConnection',
  'PostgreSQLConnection',
  'PyGreSQLConnection',
  'MySQLConnection',
  'OracleConnection',
)



##############################################
###   The connection functions


def SqliteConnection(dirname):
  '''Opens a database connection to an SQLite database using the built-in
     sqlite3 driver.  SQLite is a lightweight, disk-based database that 
     doesn't require a separate server.  It is an excellent choice for small
     applications where you want to use SQL but don't need a "real" database
     like MySQL, PostgreSQL, or Oracle.  Since it comes with Picalo, it's
     ready to go immediately -- nothing is required except a Picalo install.
     
     @param    dirname:  The directory to store database files in, or ":memory:" to keep everything in memory.
     @type     dirname:  str
     @returns: A database connection.
     @rtype:   _Connection
  '''
  assert isinstance(dirname, types.StringTypes), 'Invalid directory name: ' + str(dirname)
  func_args = {
    'dirname': dirname,
  }
  import sqlite3
  db = sqlite3.connect(dirname)
  return _SqliteConnection(db, 'SqliteConnection', func_args)


def OdbcConnection(dsn_name, username=None, password=None):
  '''Opens a database connection to an ODBC database using
     the PyODBC driver.  
     
     @param    dsn_name: The DSN string to your database (as defined in the control panel)
     @type     dsn_name: str
     @param    username: Your username for the connection
     @type     username: str
     @param    password: Your password for the connection
     @type     password: str
     @returns: A database connection.
     @rtype:   _Connection
  '''
  assert isinstance(dsn_name, types.StringTypes), 'Invalid DSN name: ' + str(dsn_name)
  assert isinstance(username, (types.StringTypes, types.NoneType)), 'Invalid username: ' + str(username)
  assert isinstance(password, (types.StringTypes, types.NoneType)), 'Invalid password: ' + str(password)
  func_args = {
    'dsn_name': dsn_name,
    'username': username,
    'password': password,
  }
  dsn = 'DSN=%s' % (dsn_name, )
  if username != None and password != None:
    dsn = 'DSN=%s;UID=%s;PWD=%s' % (dsn_name, username, password)
  elif password != None:
    dsn = 'DSN=%s;PWD=%s' % (dsn_name, password)
  import pyodbc
  db = pyodbc.connect(dsn)
  return _OdbcConnection(db, 'OdbcConnection', func_args)


def PostgreSQLConnection(database, username=None, password=None, host=None, port=None):
  '''Opens a database connection to a PostgreSQL database using
     the psycopg2 driver.
     
     @param    database: The database name to connect to.
     @type     database: str
     @param    username: Your username for the connection.
     @type     username: str
     @param    password: Your password for the connection.
     @type     password: str
     @param    host:     The server hostname or IP address.
     @type     host:     str
     @param    port:     The server port to connect on.
     @type     port:     int
     @returns: A database connection.
     @rtype:   _Connection
  '''
  assert isinstance(database, types.StringTypes), 'Invalid database name: ' + str(database)
  assert isinstance(username, (types.StringTypes, types.NoneType)), 'Invalid username: ' + str(username)
  assert isinstance(password, (types.StringTypes, types.NoneType)), 'Invalid password: ' + str(password)
  assert isinstance(host, (types.StringTypes, types.NoneType)), 'Invalid host name or IP address: ' + str(host)
  if isinstance(port, types.StringTypes):
    port = int(port)
  assert isinstance(port, (types.IntType, types.NoneType)), 'Invalid port: ' + str(port)
  func_args = {
    'database': database,
    'username': username,
    'password': password,
    'host':     host,
    'port':     port,
  }
  args = []
  args.append('dbname=%s' % (database, ))
  if username != None: args.append('user=%s' % (username, ))
  if password != None: args.append('password=%s' % (password, ))
  if host != None: args.append('host=%s' % (host, ))
  if port != None: args.append('port=%s' % (port, ))
  dsn = ' '.join(args)
  import psycopg2
  db = psycopg2.connect(dsn)
  return _Psycopg2Connection(db, 'PostgreSQLConnection', func_args)


def PyGreSQLConnection(database, username=None, password=None, host=None, port=None):
    '''Opens a database connection to a PostgreSQL database using
       the PyGreSQL driver.

       @param    database: The database name to connect to.
       @type     database: str
       @param    username: Your username for the connection.
       @type     username: str
       @param    password: Your password for the connection.
       @type     password: str
       @param    host:     The server hostname or IP address.
       @type     host:     str
       @param    port:     The server port to connect on.
       @type     port:     int
       @returns: A database connection.
       @rtype:   _Connection
    '''
    assert isinstance(database, types.StringTypes), 'Invalid database name: ' + str(database)
    assert isinstance(username, (types.StringTypes, types.NoneType)), 'Invalid username: ' + str(username)
    assert isinstance(password, (types.StringTypes, types.NoneType)), 'Invalid password: ' + str(password)
    assert isinstance(host, (types.StringTypes, types.NoneType)), 'Invalid host name or IP address: ' + str(host)
    if isinstance(port, types.StringTypes):
      port = int(port)
    assert isinstance(port, (types.IntType, types.NoneType)), 'Invalid port: ' + str(port)
    func_args = {
      'database': database,
      'username': username,
      'password': password,
      'host':     host,
      'port':     port,
    }
    args = {}
    args['database'] = database
    if username != None: args['user'] = username
    if password != None: args['password'] = password
    if host != None: 
      args['host'] = host
    else:
      args['host'] = 'localhost'
    if port != None: args['host'] += ':' + str(port)
    import pgdb
    db = pgdb.connect(**args)
    return _PyGreSQLConnection(db, 'PyGreSQLConnection', func_args)


def MySQLConnection(database, username=None, password=None, host=None, port=None):
  '''Opens a database connection to a MySQL database using
     the MySQLdb driver.
     
     @param    database: The database name to connect to.
     @type     database: str
     @param    username: Your username for the connection.
     @type     username: str
     @param    password: Your password for the connection.
     @type     password: str
     @param    host:     The server hostname or IP address.
     @type     host:     str
     @param    port:     The server port to connect on.
     @type     port:     int
     @returns: A database connection.
     @rtype:   _Connection
  '''
  assert isinstance(database, types.StringTypes), 'Invalid database name: ' + str(database)
  assert isinstance(username, (types.StringTypes, types.NoneType)), 'Invalid username: ' + str(username)
  assert isinstance(password, (types.StringTypes, types.NoneType)), 'Invalid password: ' + str(password)
  assert isinstance(host, (types.StringTypes, types.NoneType)), 'Invalid host name or IP address: ' + str(host)
  if isinstance(port, types.StringTypes):
    port = int(port)
  assert isinstance(port, (types.IntType, types.NoneType)), 'Invalid port: ' + str(port)
  func_args = {
    'database': database,
    'username': username,
    'password': password,
    'host':     host,
    'port':     port,
  }
  args = {}
  args['db'] = database
  if username != None: args['user'] = username
  if password != None: args['passwd'] = password
  if host != None: args['host'] = host
  if port != None: args['port'] = port
  import MySQLdb
  db = MySQLdb.connect(**args)
  return _MySQLdbConnection(db, 'MySQLConnection', func_args)



def OracleConnection(dsn, username=None, password=None):
  '''Opens a database connection to an Oracle database using
     the cx_Oracle driver.  
     
     @param    dsn: The DSN string to your database
     @type     dsn: str
     @param    username: Your username for the connection
     @type     username: str
     @param    password: Your password for the connection
     @type     password: str
     @returns: A database connection.
     @rtype:   _Connection
  '''
  assert isinstance(dsn, types.StringTypes), 'Invalid DSN: ' + str(dsn)
  assert isinstance(username, (types.StringTypes, types.NoneType)), 'Invalid username: ' + str(username)
  assert isinstance(password, (types.StringTypes, types.NoneType)), 'Invalid password: ' + str(password)
  func_args = {
    'dsn': dsn,
    'username': username,
    'password': password,
  }
  import cx_Oracle
  db = cx_Oracle.connect(username, password, dsn)
  return _OracleConnection(db, 'OracleConnection', func_args)



################################################################################
###   Private methods and classes in the module:                             ###
################################################################################




###################################################
###   The main database connection decorator
###   If we don't have a specialized subclass
###   for the a given database driver, this
###   superclass is used.


class _Connection(picalo.AbstractDecorator):
  '''The base class of connections'''
  
  QUERY_PARAMETER = '%s'    # the string used for parameters in queries
  
  def __init__(self, dbconnection, connect_func, connect_args):
    assert hasattr(dbconnection, 'cursor'), 'Invalid database connection: ' + str(dbconnection)
    assert hasattr(dbconnection, 'commit'), 'Invalid database connection: ' + str(dbconnection)
    picalo.AbstractDecorator.__init__(self, dbconnection)
    self._table_cache = {}   # cache to hold tables automatically loaded from the database
    self._connect_func = connect_func
    self._connect_args = connect_args
    self.filename = None
    self.modified = True
    
    
  def __getattr__(self, key):
    '''Returns the given attribute of the class.  In addition, you can type
       connection.table to access any table in the database.  The connection
       will automatically "SELECT * FROM table" and pull the records in an
       efficient way as you need them.
    '''
    # first try me and my super
    try: 
      return picalo.AbstractDecorator.__getattr__(self, key)     # first priority is this object
    except AttributeError: 
      try:
        return self.__dict__['_table_cache'][key]  # see if we already loaded this table
      except KeyError:
        useProgress = picalo.useProgress
        picalo.useProgress = False
        try:
          if key in self.list_tables():
            table = self.table("SELECT * FROM " + key) # load the table and store in the cache
            self.__dict__['_table_cache'][key] = table
            return table
          raise AttributeError, "object has no attribute '%s'" % (key, )
        finally:
          picalo.useProgress = useProgress
          
          
  def is_changed(self):
    '''Returns whether the connection needs saving'''
    return self.modified
              
          
  def refresh(self, tablename=None):
    '''Refreshes the given table name, if it has already been pulled
       from the database.  This has relation to the db.tablename syntax.  If the table
       has been updated on the database, we won't pull the new records until
       this method is called.  When tablename is None, all tables are refreshed.
       
       @param tablename:  The relation name to refresh, None to refresh all tables in this database.
       @type tablename:  str
    '''
    if tablename == None:    # refresh everything
      self.__dict__['_table_cache'] = {}
    else:                    # refresh a single table
      if self.__dict__['_table_cache'].has_key(tablename):
        del self.__dict__['_table_cache'][tablename]
          
  
  def cursor(self):
    '''Returns a cursor to the database.  Since Picalo manages cursors automatically,
       most users don't need to access them directly.   The method is provided for
       advanced users who want to explicitly manage cursors.
       
       @return:    A cursor to the database.
       @rtype:     _Cursor
    '''
    return _Cursor(self, self._decorated_object.cursor())
    
    
  def execute(self, sql, parameters=None, close_cursor=True):
    '''Runs executable queries (INSERT, UPDATE, DELETE) that don't 
       return tables.  This is the primary methods to use when posting information
       to a database.
       
       IMPORTANT: Since most databases don't commit changes automatically, you *must* call 
       commit() to make the changes permanent.  If you close the connection without
       committing, you'll lose all of your changes.
       
       Alternatively, you can call rollback() to undo all changes since the last commit()
       call.
       
       Example:
         >>> myconn.execute("CREATE TABLE mytable (id integer, name varchar(20))")
         >>> myconn.execute("INSERT INTO mytable (id, name) VALUES (1, 'Dennis')")
         >>> myconn.commit()
       
       @param sql:           The SQL string to execute.
       @type  sql:           string
       @param parameters:    The parameters to be sent to the database.
       @type  parameters:    List or Tuple
       @param close_cursor:  Determines whether the cursor is closed automatically.  Unless you need the cursor for something else, you should allow it to close automatically.
       @type  close_cursor:  bool
       @return:              The return value from the database.  This varies from database to database and is probably not very useful.  If errors occur in the operation, exceptions are thrown.
       @rtype:               object
    '''
    # types are checked in cursor.execute
    cursor = self.cursor()
    try:
      if parameters != None:
        ret = cursor.execute(sql, parameters)
      else:
        ret = cursor.execute(sql)
    finally:
      if close_cursor:
        cursor.close()
    return ret
    
  
  def commit(self):
    '''Commits any changes to the database.  If you make modifications of any
       kind to your database tables (INSERT, UPDATE, CREATE, etc.), you *must*
       commit those changes by calling commit().
       
       If you close a database connection without committing, you'll lose all
       changes.
       
       Analyses that only run SELECT calls on a database do not need to commit().
       
       See the execute() method for an example of committing.
    '''
    self._decorated_object.commit()
    

  def query(self, sql, parameters=None):
    '''Runs a SELECT query that returns a result set.  This method provides
       efficient access to extremely large data sets, but it is not as easy to
       use as table().  Most users should use table() unless you explicitly need to
       access only one record at a time. In other words, check out 
       Database.Connection.table(sql) and use it unless you really need
       the efficiency offered by this method.
       
       Stated differently, query() and table() accomplish the same task:
       query a database.  The query() method is more efficient and the table()
       method is more powerful and easier to use.
       
       If you need to efficiently use memory and must use this method, you should use an
       iterator in a for loop as is done in the example.  This allows you
       to access one record at a time.  Note that you can only go in forward
       direction one at a time.  If you need to go backwards in your analysis,
       either store the records in variables or use table().
       
       Example:
         >>> import psycopg
         >>> myconn = Database.Connection(psycopg.connect(database='mydb'))
         >>> for record in myconn.query("SELECT * FROM mytable"):
         ...   print record['id']
         ...   print record['name']
         ...
         1
         Dennis
       
       @param sql:  The SQL string to execute.
       @type  sql:  string
       @param parameters:    The parameters to be sent to the database.
       @type  parameters:    List or Tuple
       @return:     A cursor to the result set.
       @rtype:      Cursor
    '''
    # types are checked in cursor.query
    return self.cursor().query(sql, parameters)
    

  def query1(self, sql, parameters=None):
    '''Runs a SELECT query and returns only the first record from the result set.
       There are many times when you expect only a single record in the result
       set, such a key value query.  In these circumstances, query1() provides
       easy access directly to the first record.
       
       The method returns a single Record instance or None if no results are found.

       Example:
         >>> import psycopg
         >>> myconn = Database.Connection(psycopg.connect(database='mydb'))
         >>> rec = myconn.query1("SELECT min(id) FROM mytable")
         >>> print rec
         1
       
       @param sql:  The SQL string to execute.
       @type  sql:  string
       @param parameters:    The parameters to be sent to the database.
       @type  parameters:    List or Tuple
       @return:     A single result set record.
       @rtype:      Record
    '''
    # types are checked in cursor.query1
    return self.cursor().query1(sql, parameters)

    
  def table(self, sql, parameters=None):
    '''This method is the primary access to databases within Picalo.  Most users will create
       a connection and then call only this method to retrieve data from tables.
    
       Retrieves a Table to the results of a new query or the most recently-run query.
       Table objects have additional methods not found in this module, such as
       pretty printing, saving to text files, and mutability.  
       
       If your database driver is one of the enhanced support drivers (such as 
       PythonWin's odbc, psycopg, and MySQLdb drivers), the method will automatically
       set column types.
       
       The primary reason you would not want to use this method is that it loads
       all records into memory at query time, so it might use up your memory
       with super large tables.       
       Example:
        >>> import psycopg
        >>> myconn = Database.Connection(psycopg.connect(database='conan'))
        >>> data = myconn.table("SELECT * FROM mytable")
        >>> data.view()
        +----+--------+
        | id |  name  |
        +----+--------+
        |  1 | Dennis |
        +----+--------+
        
       @param sql:   The SQL string to execute.
       @type  sql:   string
       @param parameters:    The parameters to be sent to the database.
       @type  parameters:    List or Tuple
       @return:      A picalo table containing the results.
       @rtype:       Table
    '''
    # types are checked in cursor.table
    return self.cursor().table(sql, parameters)
    
    
  def insert_query_builder(self, tablename):
    '''This method returns a helper class for creating INSERT queries.
       It allows you to create INSERT queries piece by piece and is 
       available for advanced users.  It is useful when creating an SQL
       query programatically in a script.
    
       Example:
       >>> db = Database.PostgreSQLConnection('mydb')
       >>> q = db.insert_query_builder('test')
       >>> q.add('id', 5)
       >>> q.add('name', 'Sally')
       >>> print q
       QueryBuilder: <cursor>.execute("INSERT INTO test (id, name) VALUES (%s, %s)", [5, 'Sally'])
       >>> q.execute()
       >>> db.commit()

       @param tablename:   The database table name to insert the records into
       @type  tablename:   str
       @return:            An InsertQueryBuilder object.  
       @rtype:             InsertQueryBuilder
    '''
    assert isinstance(tablename, types.StringTypes), 'Invalid table name: ' + str(tablename)
    return InsertQueryBuilder(tablename, self)
    
    
  def update_query_builder(self, tablename):
    '''This method returns a helper class for creating simple UPDATE queries.
       It allows you to create UPDATE queries piece by piece and is 
       available for advanced users.  It is useful when creating an SQL
       query programatically in a script.

       Example:    
       >>> db = Database.PostgreSQLConnection('mydb')
       >>> q = db.update_query_builder('test')
       >>> q.add_where('id', 3)
       >>> q.add('name', 'newname')
       >>> print q
       QueryBuilder: <cursor>.execute("UPDATE test SET name=%s WHERE id=%s", ['newname', 3])
       >>> q.execute()     
       >>> db.commit() 
       
       @param tablename:   The database table name to update the records in.
       @type  tablename:   str
       @return:            An UpdateQueryBuilder object.  
       @rtype:             UpdateQueryBuilder
    '''
    assert isinstance(tablename, types.StringTypes), 'Invalid table name: ' + str(tablename)
    return UpdateQueryBuilder(tablename, self)
  
  
  def select_query_builder(self, tablename, select_fields=["*"]):
    '''This method returns a helper class for creating simple SELECT queries.
       It allows you to create SELECT queries piece by piece and is 
       available for advanced users.  It is useful when creating an SQL
       query programatically in a script.

       Example:    
       >>> db = Database.PostgreSQLConnection('mydb')
       >>> q = db.select_query_builder('test', ['id', 'name'])
       >>> q.add_where('id', 1)
       >>> print q
       QueryBuilder: <cursor>.execute("SELECT id, name FROM test WHERE id=%s", [1])
       >>> results = q.table()
       >>> results.prettyprint()
       +----+--------+
       | id |  name  |
       +----+--------+
       |  1 | Joseph |
       +----+--------+

       @param tablename:     The database table name to update the records in.
       @type  tablename:     str
       @param select_fields: The fields to be selected by the query.
       @type  select_fields: list or tuple of strings
       @return:              An SelectQueryBuilder object.  
       @rtype:               SelectQueryBuilder
    '''
    assert isinstance(tablename, types.StringTypes), 'Invalid table name: ' + str(tablename)
    return SelectQueryBuilder(tablename, select_fields, self)
 
 
  def _create_database_table(self, table, name, replace=False):
    '''Creates a databaes relation matching the column names and types
       of the given table.'''
    check_valid_table(table)
    assert isinstance(name, types.StringTypes), 'Invalid table name: ' + str(name)
    # try to drop the table if it exists and the user asked for replace
    if replace:
      try:
        try:
          cursor = self.cursor()
          cursor.execute('DROP TABLE ' + name)
        finally:
          cursor.close()
      except:  # if the table doesn't exist, we get an error
        pass
    self.commit()  # end the current transaction, just in case we had a failure (some db's like postgresql require this)
        
    # create the table by automatically figuring out the field types
    cols = []
    tablecolumns = table.get_columns()
    for column in tablecolumns:
      typ = column.get_type()
      try:
        dbtype = picalo.TYPE_TO_DB[typ]
        cols.append(column.name + ' ' + dbtype)
      except (AttributeError, KeyError):  # a string type, so figure out how big
        if len(column) > 0:
          length = max(1, max([ len(str(val)) for val in column ]))
        else:
          length = 100
        if length <= 255:
          cols.append(column.name + ' varchar(' + str(length) + ')')
        else:
          cols.append(column.name + ' text')
    try:
      cursor = self.cursor()
      cursor.execute('CREATE TABLE ' + name + ' (' + ', '.join(cols) + ')')
    finally:
      self.commit()
      cursor.close()
    self.commit()
  

  def post_table(self, table, name, replace=False, add_if_empty=False):
    '''Creates a new table in the database and posts the records
       in the given table.  If a table with the given name
       already exists in the table, the method throws an error unless
       the "replace" option is True.  
       
       This is a convenience method provided to make importing of data from
       text files (CSV, TSV, etc.) and other sources into databases easy.
       The example shows how a CSV file can be posted to database in
       just two lines of code.  
       
       The method always tries to create a new table in
       the database -- it will not append records into an existing table.
       If the table cannot be created (if it exists, for example), an 
       error is thrown.  If the "replace" option is True, any
       existing tables by this name are dropped before the new table
       is created. 
       
       The method automatically commits the data to the database.  You cannot
       use rollback after this method has finished.  This is required by
       some databases after creation of a table, so it has to be done.
       
       Example:
        >>> myconn = Database.PostgreSQLConnection('mydb')
        >>> data = Table([('id', int), ('name', unicode)],[
        ...   [ 1, 'Benny' ],
        ...   [ 2, 'Vijay' ],
        ... ])
        >>> myconn.post_table(data, 'mytable', True)
        >>> myconn.table('SELECT * FROM mytable').view()         
        +----+-------+
        | id |  name |
        +----+-------+
        |  1 | Benny |
        |  2 | Vijay |
        +----+-------+
  
       @param table:        The picalo table to post
       @type  table:        Table
       @param name:         The new database table name
       @type  name:         str
       @param replace:      Whether to replace any existing database tables with the given name.  If the database table exists and replace is False (the default), the method will throw an error.
       @type  replace:      bool
       @param add_if_empty: Whether to add fields that are empty (None or '') or to simply leave them out of the query.
       @type  add_if_empty: bool
    '''
    # create the new table
    self._create_database_table(table, name, replace)
    
    # add the records
    try:
      tablecolumns = table.get_columns()
      for recindex in range(len(table)):
        show_progress('Uploading records...', float(recindex) / float(len(table)))
        rec = table[recindex]
        sql = self.insert_query_builder(name)
        for i in range(len(tablecolumns)):
          sql.add(tablecolumns[i].name, rec[i], add_if_empty=add_if_empty)
        sql.execute()
        self.commit()
    finally:
      self.commit()
      clear_progress()
      
      
  def copy_table(self, source_connection, source_tablename, dest_tablename, replace=False):
    '''Copies a table from a database into this connection.  This method pulls records one by one
       from the source connection and posts them to this connection.  It
       automatically creates the new destination table, optionally replacing any existing
       tables with the name.
       
       This is a convenience method provided to make importing of data from
       other databases easy.  This method is different from post_table because it
       is more memory-efficient.  The post_table method requires that you have an
       existing Picalo Table, which means all of the data needs to be in memory at
       once.  This method copies record by record, meaning it only needs to hold one
       record at a time.  This method works only with database connections, not with 
       CSV or other types of sources.
       
       The method always tries to create a new table in
       the database -- it will not append records into an existing table.
       If the table cannot be created (if it exists, for example), an 
       error is thrown.  If the "replace" option is True, any
       existing tables by this name are dropped before the new table
       is created. 
       
       The method automatically commits the data to the database.  You cannot
       use rollback after this method has finished.  This is required by
       some databases after creation of a table, so it has to be done.
    '''
    # check data types
    assert isinstance(source_connection, _Connection), 'The source_connection parameter must be a valid Picalo database connection'
    assert isinstance(source_tablename, types.StringTypes), 'The source table name parameter must be a string.'
    assert source_tablename != '', 'Please enter a table name for the source table name.'
    assert isinstance(dest_tablename, types.StringTypes), 'The destination tablename parameter must be a string.'
    assert source_tablename != '', 'Please enter a table name for the destination table name.'
    
    # query the data for this table
    source_cursor = source_connection.cursor()
    try:
      table = source_cursor.table("SELECT * FROM %s" % (source_tablename, ))
    
      # create the new table
      self._create_database_table(table, dest_tablename, replace)
    
      # add the records.  this method takes advantage of the fact that database-based tables
      # are loaded just in time.  since the code pulls the record data directly from the
      # cursor rather than the table variable, the data is never cached in the table object.
      try:
        for recindex in range(len(table)):
          show_progress('Copying records...', float(recindex) / float(len(table)))
          rec = table[recindex]
          sql = self.insert_query_builder(dest_tablename)
          for i in range(table.column_count()):
            sql.add(table.column(i).name, rec[i], add_if_empty=True)
          sql.execute()
          self.commit()
      finally:
        self.commit()
        clear_progress()
    finally:
      source_cursor.close()    


  def __str__(self):
    '''Returns a string representation of this connection'''
    return repr(self)


  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._Connection decorator>: ' + repr(self._decorated_object)


  def save(self, filename):
    '''Saves the database connection'''
    # actual function def is below the classes in this file
    save(self, filename)
    
  
  #########   Specialized methods that subclasses should override   ##################

  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    pass
    
  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    return []
    
    
  def _get_columns(self, tablename):
    '''Returns a list of the columns in the given table.  Subclasses may have a more efficient way to do this.'''
    cursor = self.cursor()
    cursor.execute("SELECT * FROM " + tablename)
    colnames = [ row[0] for row in cursor.description ]
    cursor.close()
    return colnames


        
    
    
#########################################################################
###   Specialization classes for specific databases


class _SqliteConnection(_Connection):
  '''A specialization class for sqlite3 connections'''
  
  QUERY_PARAMETER = '?'  
  
  def __init__(self, dbconnection, func, args):
    _Connection.__init__(self, dbconnection, func, args)
    self._tablenames = None

  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._SqliteConnection decorator>: ' + repr(self._decorated_object)

  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    # how to do this?  Sqlite doesn't give me the column types in the cursor description list!
    pass
    

  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    if refresh or self._tablenames == None:
      self._tablenames = []
      try:
        cursor = self.cursor()
        self._tablenames = [ row[0] for row in cursor.query("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name") ]
        cursor.close()
      except:  # thrown when connection is closed
        pass
    return self._tablenames


class _Psycopg2Connection(_Connection):
  '''A specialization class for psycopg2 connections'''
  def __init__(self, dbconnection, func, args):
    _Connection.__init__(self, dbconnection, func, args)
    self._tablenames = None
  
  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._Psycopg2Connection decorator>: ' + repr(self._decorated_object)

  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    for i in range(len(cursor.description)):
      typecode = cursor.description[i][1]
      if typecode in ( 1114, 1184, 704, 1186, ): # date time types
        table.set_type(i, DateTime)
      elif typecode in ( 21, 23, ): # integer
        table.set_type(i, int)
      elif typecode in ( 20, ):  # long integer
        table.set_type(i, long)
      elif typecode in ( 701, 700, 1700, ): # float
        table.set_type(i, number)
        table.set_format(i, '"%0.4f" % value')


  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    if refresh or self._tablenames == None:
      self._tablenames = []
      try:
        cursor = self.cursor()
        self._tablenames = [ row[0] for row in cursor.query("select tablename from pg_tables where schemaname='public'") ]
        cursor.close()
      except:  # thrown when connection is closed
        pass
    return self._tablenames
    

  def _get_columns(self, tablename):
    '''Returns a list of the columns in the given table'''
    cursor = self.cursor()
    columns = [ row[0] for row in cursor.query("select column_name from information_schema.columns where table_name='%s'" % (tablename, )) ]
    cursor.close()
    return columns
    


    
class _PyGreSQLConnection(_Connection):
  '''A spoecialization class for PyGreSQL connections'''    
  def __init__(self, dbconnection, func, args):
    _Connection.__init__(self, dbconnection, func, args)
    self._tablenames = None

  
  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._PyGreSQLConnection decorator>: ' + repr(self._decorated_object)


  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    for i in range(len(cursor.description)):
      typecode = cursor.description[i][1]
      if typecode in ( 'abstime', 'reltime', 'tinterval', 'date', 'time', 'timespan', 'timestamp', 'timestamptz', 'interval', ): # date time types
        table.set_type(i, DateTime)
      elif typecode in ( 'int2', 'int4', 'serial', ): # integer
        table.set_type(i, int)
      elif typecode in ( 'int8', ):  # long integer
        table.set_type(i, long)
      elif typecode in ( 'float4', 'float8', 'numeric', 'money' ): # float
        table.set_type(i, number)
        table.set_format(i, '"%0.4f" % value')
      elif typecode in ( 'bool', ): # boolean
        table.set_type(i, boolean)


  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    if refresh or self._tablenames == None:
      self._tablenames = []
      try:
        cursor = self.cursor()
        self._tablenames = [ row[0] for row in cursor.query("select tablename from pg_tables where schemaname='public'") ]
        cursor.close()
      except:  # thrown when connection is closed
        pass
    return self._tablenames
    

  def _get_columns(self, tablename):
    '''Returns a list of the columns in the given table'''
    cursor = self.cursor()
    columns = [ row[0] for row in cursor.query("select column_name from information_schema.columns where table_name='%s'" % (tablename, )) ]
    cursor.close()
    return columns
    

    

class _OdbcConnection(_Connection):
  '''A specialization class for PyODBC connections'''
  
  QUERY_PARAMETER = '?'  # PyODBC has a special one (should be %s per spec)
  
  def __init__(self, dbconnection, func, args):
    _Connection.__init__(self, dbconnection, func, args)
    self._tablenames = None

  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._OdbcConnection decorator>: ' + repr(self._decorated_object)

  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    for i in range(len(cursor.description)):
      typecode = cursor.description[i][1]
      size = cursor.description[i][3]
      precision = cursor.description[i][4]
      if typecode == datetime.datetime:
        table.set_type(i, DateTime)
      elif typecode == types.IntType:
        table.set_type(i, int)
      elif typecode == types.FloatType:
        table.set_type(i, number)
        table.set_format(i, '"%0.4f" % value')

  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    if refresh or self._tablenames == None:
      self._tablenames = []
      try:
        cursor = self.cursor()
        self._tablenames = [ row[2] for row in cursor.tables() if row[3] == 'TABLE' ]
        cursor.close()
      except:  # thrown when connection is closed
        pass
    return self._tablenames


  def _get_columns(self, tablename):
    '''Returns a list of the columns in the given table'''
    cursor = self.cursor()
    colnames = [ row[3] for row in cursor.columns(table=tablename) ]
    cursor.close()
    return colnames
    

class _MySQLdbConnection(_Connection):
  '''A specialization class for MySQLdb connections'''
  def __init__(self, dbconnection, func, args):
    _Connection.__init__(self, dbconnection, func, args)
    self._tablenames = None

  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._MySQLdbConnection decorator>: ' + repr(self._decorated_object)

  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    for i in range(len(cursor.description)):
      typecode = cursor.description[i][1]
      if typecode in ( 12, 10, 13, 14, ): # date time types
        table.set_type(i, DateTime)
      elif typecode in ( 1, 2, 3, ): # integer
        table.set_type(i, int)
      elif typecode in ( 8, 9, ):  # long integer
        table.set_type(i, long)
      elif typecode in ( 4, 5, 0, ): # float
        table.set_type(i, number)
        table.set_format(i, '"%0.4f" % value')


  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    if refresh or self._tablenames == None:
      self._tablenames = []
      try:
        cursor = self.cursor()
        self._tablenames = [ row[0] for row in cursor.query("show tables") ]
        cursor.close()
      except:  # thrown when connection is closed
        pass
    return self._tablenames


class _OracleConnection(_Connection):
  '''A specialization class for cx_Oracle connections'''
  def __init__(self, dbconnection, func, args):
    _Connection.__init__(self, dbconnection, func, args)
    self._tablenames = None

  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._OracleConnection decorator>: ' + repr(self._decorated_object)

  def _set_table_types(self, cursor, table):
    '''Helper method for _Connection.table().  After the query is run, this method
       uses the cursor to set the column types appropriately.'''
    import cx_Oracle
    for i in range(len(cursor.description)):
      typecode = cursor.description[i][1]
      if typecode in ( cx_Oracle.DATETIME, cx_Oracle.TIMESTAMP ): # date time types
        table.set_type(i, DateTime)
      elif typecode in ( cx_Oracle.NUMBER, ): # integer
        if cursor.description[i][5] == 0:  # scale?
          table.set_type(i, long)
        else:
          table.set_type(i, number)
          table.set_format(i, '"%0.4f" % value')

  def list_tables(self, refresh=True):
    '''Returns the table names in this database as a list'''
    if refresh or self._tablenames == None:
      self._tablenames = []
      try:
        cursor = self.cursor()
        self._tablenames = [ row[0] for row in cursor.query("SELECT table_name FROM user_tables ORDER BY table_name") ]
        cursor.close()
      except:  # thrown when connection is closed
        pass
    return self._tablenames  
    
    
    

##############################################################
###   Our cursor decorator.  It is created automatically
###   by _Connection.cursor() and other methods in
###   _Connection.


class _Cursor(picalo.AbstractDecorator):
  '''A database cursor.  Since Picalo automatically manages cursors,
     you don't need to create these directly.
  '''
  def __init__(self, parent, obj):
    picalo.AbstractDecorator.__init__(self, obj)
    self.parent = parent
    self.columns_map = {}
  
  def execute(self, sql, parameters=None):
    '''Executes the given sql'''
    assert isinstance(sql, types.StringTypes), 'Invalid SQL string: ' + str(sql)
    if parameters == None:
      self._decorated_object.execute(sql)
    else:
      self._decorated_object.execute(sql, parameters)
    if self.description == None:
      self.columns_map = {}
    else:
      self.columns_map = dict([ (self.description[i][0], i) for i in range(len(self.description)) ])
  

  def query(self, sql, parameters=None):
    '''Runs an SQL query.  The results of this method can be iterated directly.  Although you cannot
       randomly access the records by index (only sequentially in a for loop), this method is 
       much more efficient than table() because it pulls only one record into memory at a time.'''
    # types are checked in execute
    try:
      show_progress('Running query...', 0.0)
      self.execute(sql, parameters)
      return self
    finally:
      clear_progress()
    

  def query1(self, sql, parameters=None):
    '''Runs an SQL query (using cursor.execute) and returns the first record
       in the result set.  This is useful when you only expect one row (such as
       SELECT MAX(...) queries).
    '''
    # types are checked in execute
    try:
      show_progress('Running query...', 0.0)
      self.query(sql, parameters)
      return self.fetchone()
    finally:
      clear_progress()
    

  def table(self, sql, parameters=None):
    '''See Database.Connection.table() for information on this method.'''
    query = Query(sql, parameters)
    query.execute(self)
    return query.table
        
    
  def __iter__(self):
    '''Returns an iterator to the most recent results'''
    while True:
      rec = self.fetchone()
      if rec == None:
        return
      yield rec    
    

  def __str__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._Cursor decorator>: ' + str(self._decorated_object)
    

  def __repr__(self):
    '''Returns a string representation of this connection'''
    return '<Picalo Database._Cursor decorator>: ' + repr(self._decorated_object)



########################################################
###  Query extension to Table to differentiate the two
    
#class Query(Table):
#  '''Extension to table to load a query'''
#  def __init__(self,     
    
    

################################################
###   Classes to help in creating SQL queries


class QueryBuilderException(Exception):
  '''Simple exception to report query builder problems, with an embedded error.'''
  def __init__(self, e, msg):
    self.e = e
    self.msg = msg
    
  def __repr__(self):
    return 'Error in query: %s; %s' % (self.msg, repr(self.e))
    
  def __str__(self):
    return repr(self)
    

class _BaseQueryBuilder:
  '''The base of query builder with common code.'''
  def __init__(self, tablename, connection):
    self._tablename = tablename
    self._db = connection
    self._fieldnames = []
    self._values = []
    self._wherenames = []
    self._wherevalues = []

  
  def add(self, field, value, add_if_empty=True):
    """
    Adds a field/value pair to the statement,
    optionally skipping the add if the value is None or empty ('').
    This method is only valid for update-type queries and is ignored
    in select queries.
    """
    if not add_if_empty and (value == '' or field == '' or value == None or field == None):
      return
    if field in self._fieldnames:
      index = self._fieldnames.index(field)
      self._values[index] = value
    else:
      self._fieldnames.append(field)
      self._values.append(value)
      
      
  def add_where(self, field, value, add_if_empty=True):
    '''Adds a field/value pair to the statement for the WHERE
       clause.  This method is only used in statements where this
       makes sense (select, update).  It is ignored in insert queries.
    '''
    if not add_if_empty and (value == '' or field == '' or value == None or field == None):
      return
    if field in self._wherenames:
      index = self._wherenames.index(field)
      self._wherevalues[index] = value
    else:
      self._wherenames.append(field)
      self._wherevalues.append(value)
          
      
  def __setitem__(self, field, value):
    '''Allows dictionary-use of the object to call the add method.
       It always adds the value, even if it is None or empty.
    '''
    self.add(field, value)
    
    
  def __getitem__(self, field):
    '''Returns the item with the given field name'''
    return self.get(field)
    
    
  def get(self, field, default=None):
    '''Returns the item with the given field name, or the default value if none exists'''
    if field in self._fieldnames:
      index = self._fieldnames.index(field)
      return self._values[index]
    return default
    

  def execute(self):
    '''Executes the SQL statement.  This method should be called for 
       insert and update queries.
       Does *not* commit if transactions are being used.
    '''
    cursor = self._db.cursor()
    try:
      cursor.execute(self.get_query_string(), self.get_parameters())
    except Exception, e:
      raise QueryBuilderException(e, str(self))
    cursor.close()


  def get_query_string(self):
    '''Returns the query string as it will be sent to the database.'''
    raise NotImplementedError, 'Subclass of _BaseQueryBuilder did not implement required method get_query_string'


  def get_parameters(self):
    '''Returns the parameters as they will be sent to the database'''
    raise NotImplementedError, 'Subclass of _BaseQueryBuilder did not implement required method get_parameters'


  def __len__(self):
    '''Returns the number of fields in this query'''
    return len(self._fieldnames)
    
    
  def _format(self, st):
    '''Helper method to ensure that special characters are escaped correctly'''
    return str(st).replace('"', '\\"')    
    
    
  def __str__(self):
    return '<cursor>.execute("' + self.get_query_string() + '", ' + str(self.get_parameters()) + ")"



class UpdateQueryBuilder(_BaseQueryBuilder):
  """Helps in building a simple update SQL call.  This class is useful
     when creating SQL from a script, piece by piece.  See 
     _Connection.update_query_builder() for more information."""
  def __init__(self, tablename, connection):
    _BaseQueryBuilder.__init__(self, tablename, connection)
    

  def get_query_string(self):
    '''Returns the query string as it will be sent to the database.'''
    sql = []
    sql.append("UPDATE " + self._format(self._tablename))
    if len(self._fieldnames) > 0:
      params = [ self._format(name) + "=" + self._db.QUERY_PARAMETER for name in self._fieldnames ]
      sql.append(" SET " + ', '.join(params))
    if len(self._wherenames) > 0:
      where = [ self._format(name) + "=" + self._db.QUERY_PARAMETER for name in self._wherenames ]
      sql.append(" WHERE " + ', '.join(where))
    return ''.join(sql)
    
  
  def get_parameters(self):  
    '''Returns the parameters as they will be sent to the database'''
    params = []
    for val in self._values + self._wherevalues:
      if isinstance(val, Date):
        params.append(val.strftime('%Y-%m-%d'))
      elif isinstance(val, DateTime):
        params.append(val.strftime('%Y-%m-%d %H:%M:%S'))
      else:
        params.append(val)
    return params
    

class InsertQueryBuilder(_BaseQueryBuilder):
  """Helps in building a simple insert SQL call.  This class is useful
     when creating SQL from a script, piece by piece.  See 
     _Connection.insert_query_builder() for more information."""
  def __init__(self, tablename, connection):
    _BaseQueryBuilder.__init__(self, tablename, connection)
    

  def get_query_string(self):
    '''Returns the query string as it will be sent to the database.'''
    sql = []
    sql.append("INSERT INTO " + self._format(self._tablename))
    if len(self._fieldnames) > 0:
      sql.append(' (')
      sql.append(', '.join([ self._format(name) for name in self._fieldnames ]))
      sql.append(') VALUES (')
      sql.append(', '.join([self._db.QUERY_PARAMETER for val in self._values]))
      sql.append(')')
    return ''.join(sql)
    
  
  def get_parameters(self):  
    '''Returns the parameters as they will be sent to the database'''
    params = []
    for val in self._values:
      if isinstance(val, Date):
        params.append(val.strftime('%Y-%m-%d'))
      elif isinstance(val, DateTime):
        params.append(val.strftime('%Y-%m-%d %H:%M:%S'))
      else:
        params.append(val)
    return params
    


class SelectQueryBuilder(_BaseQueryBuilder):
  """Helps in building a simple select SQL call.  This class is useful
     when creating SQL from a script, piece by piece. See 
     _Connection.select_query_builder() for more information."""
  def __init__(self, tablename, select_fields, connection):
    _BaseQueryBuilder.__init__(self, tablename, connection)
    assert isinstance(select_fields, (types.TupleType, types.ListType)), 'The select_fields parameter must be a list or tuple of field names (or *) to select from the table.'
    assert len(select_fields) > 0, 'The select_fields parameter must be a list or tuple of field names (or *) to select from the table.  You must provide at least one field name.'
    self.select_fields = select_fields
    

  def get_query_string(self):
    '''Returns the query string as it will be sent to the database.'''
    sql = []
    sql.append("SELECT ")
    sql.append(', '.join([ self._format(name) for name in self.select_fields ]))
    sql.append(" FROM " + self._format(self._tablename))
    if len(self._wherenames) > 0:
      where = [ self._format(name) + "=" + self._db.QUERY_PARAMETER for name in self._wherenames ]
      sql.append(" WHERE " + ', '.join(where))
    return ''.join(sql)
    

  def get_parameters(self):  
    '''Returns the parameters as they will be sent to the database'''
    params = []
    for val in self._wherevalues:
      if isinstance(val, Date):
        params.append(val.strftime('%Y-%m-%d'))
      elif isinstance(val, DateTime):
        params.append(val.strftime('%Y-%m-%d %H:%M:%S'))
      else:
        params.append(val)
    return params
    

  def query(self):
    '''Runs the query and returns a cursor to the results data set'''
    cursor = self._db.cursor()
    return cursor.query(self.get_query_string(), self.get_parameters())

    
  def query1(self):
    '''Runs the query and returns a single record (i.e. tuple) of the first result.
       This is useful when you are sure you will get only one result back from
       the query (such as a COUNT operator).'''
    cursor = self._db.cursor()
    return cursor.query1(self.get_query_string(), self.get_parameters())
    
    
  def table(self):
    '''Runs the query and returns a Picalo table containing the results
       of the query.'''
    cursor = self._db.cursor()
    return cursor.table(self.get_query_string(), self.get_parameters())
    


############################################
###   Load and save routines

# pickle protocol to use
PICKLE_PROTOCOL = 2


def load(filename):
  '''Loads the connection from the given filename.
  
     @param filename: The filename to load from.  This can also be an open stream.
     @type  filename: str
  '''
  fileopened = False
  if type(filename) in types.StringTypes:
    filepath = filename
    filename = open(filename, 'rb')
    fileopened = True
    
  # ungzip it on the fly (if it is not already a GzipFile)
  startedzip = False
  gin = filename
  if not isinstance(gin, gzip.GzipFile):
    gin = gzip.GzipFile(fileobj=gin, mode='rb')
    startedzip = True
      
  # read the data and create the connection
  data = pickle.load(gin)
  func = globals()[data['connect_func']]
  conn = func(**data['connect_args'])
  conn.modified = False
  
  # close up shop and return
  if startedzip:
    gin.close() 
  if fileopened:
    conn.filename = filepath
    filename.close()
  return conn
  
  
def save(conn, filename):
  '''Saves the given connection to the filename.
  
     @param conn:     The database connection to save.
     @type  conn:     Picalo Database Connection
     @param filename: The filename to save to.  This can also be an open stream.
     @type  filename: str
  '''
  assert isinstance(conn, _Connection), 'Only database connections opened with Picalo can be saved with this method.'
  
  # get a file pointer
  fileopened = False
  if type(filename) in types.StringTypes:
    conn.filename = filename
    filename = open(filename, 'wb')
    fileopened = True
  
  # gzip it on the fly (if it is not already a GzipFile)
  openedzip = False
  gout = filename
  if not isinstance(gout, gzip.GzipFile):
    gout = gzip.GzipFile(fileobj=gout, mode='wb')
    openedzip = True
    
  # save the database connection information
  data = {
    'connect_func': conn._connect_func,
    'connect_args': conn._connect_args,
  }
  pickle.dump(data, gout, PICKLE_PROTOCOL)
  
  # close up shop
  gout.flush()
  if openedzip:
    gout.close()
  if fileopened:
    filename.close()
  conn.modified = False
  
  
  
  
################################################
###   Query object

class Query:
  '''Represents a query on a database.'''
  def __init__(self, sql, parameters=None, conn=None):
    '''Creates a new query object.
    
       @param sql:           The SQL string to execute.
       @type  sql:           string
       @param parameters:    The parameters to be sent to the database.
       @type  parameters:    List or Tuple
       @param conn:          If not None, the query is immediately executed with the given connection.  This parameter can also be a Picalo _Cursor object.
       @type  conn:          Database._Connection or Database._Cursor object
    '''
    assert sql, 'The sql parameter cannot be empty.'
    assert isinstance(sql, types.StringTypes), 'The sql parameter must be an SQL string.'
    self._connect_func = None
    self._connect_args = None
    self.filename = None
    self.table = None
    self.set_sql(sql, parameters, conn)  # adds more class variables, see this method


  def set_sql(self, sql, parameters=None, conn=None):
    '''Modifies the sql of this connection, potentially re-executing it with the given connection'''
    self._modified = True
    self._sql = sql
    self._parameters = parameters
    if conn != None:
      self.execute(conn)
    
    
  def get_sql(self):
    '''Returns the SQL for this query'''
    return self._sql


  def execute(self, conn):
    '''Run the query using the given database connection.
    
       @param conn:          The connection to execute the query on.  This parameter can also be a Picalo _Cursor object.
       @type  conn:          Database._Connection or Database._Cursor object
    '''
    # check the parameter
    assert isinstance(conn, (_Connection, _Cursor)), 'The conn parameter must be a Picalo connection (or cursor) object'
    if isinstance(conn, _Connection):
      cursor = conn.cursor()
    else:
      cursor = conn
    
    # save the database connection information.  This helps the Picalo GUI
    # match queries to database connections when queries are run.
    if self._connect_func != cursor.parent._connect_func:
      self.modified = True
      self._connect_func = cursor.parent._connect_func
    if self._connect_args != cursor.parent._connect_args:
      self.modified = True
      self._connect_args = cursor.parent._connect_args
    
    # run the query
    cursor.query(self._sql, self._parameters)
    assert cursor.description != None, 'You must first run a query that returns results to create a Table object!'
    
    # if the table currently exists, notify that it will be changing
    if self.table != None:
      self.table._notify_listeners(level=3)
    
    # first create the table
    self.table = Table(picalo.make_unique_colnames(picalo.ensure_valid_variables([ desc[0] for desc in cursor.description ])))
    
    # set the types
    cursor.parent._set_table_types(cursor, self.table)
    
    # set up the table data
    for row in cursor:
      self.table.append(row)
    
  
  def is_changed(self):
    '''Returns whether the table has been changed since loading'''
    return self._modified
    
    
  def save(self, filename):
    '''Saves the query to the given filename'''
    save_query(self, filename)
    
    
  def view(self):
    '''Opens a spreadsheet-view of the table if Picalo is being run in GUI mode.
       If Picalo is being run in console mode, it redirects to prettyprint().
       This is the preferred way of viewing the data in a table.
    '''
    # this is explicitly included in the class so it runs even if the table
    # hasn't been loaded.  the GUI will execute automatically.
    picalo.TableModule.view(self)
    

  def __eq__(self, other):
    '''Returns whether this query is equal to another query (or list of lists).'''
    if isinstance(other, (Query)):
      for name in [ '_sql', '_parameters', '_connect_func', '_connect_args' ]:
        if getattr(self, name) != getattr(other, name):
          return False
      return True
    return False
    
    
    
  ################################################
  ###   Pass-through most methods to Table    
    
  def __getattr__(self, name):
    # class methods are already searched before __getattr__ gets called (not true with __setattr__, btw)
    assert self.table != None, 'You must first execute the query before calling "%s"' % (name, )
    return getattr(self.table, name)
    
    
##########################################
###   Load and save methods for query
def load_query(filename):
  '''Loads the query from the given filename.
  
     @param filename: The filename to load from.  This can also be an open stream.
     @type  filename: str
  '''
  fileopened = False
  if type(filename) in types.StringTypes:
    filepath = filename
    filename = open(filename, 'rb')
    fileopened = True
    
  # ungzip it on the fly (if it is not already a GzipFile)
  startedzip = False
  gin = filename
  if not isinstance(gin, gzip.GzipFile):
    gin = gzip.GzipFile(fileobj=gin, mode='rb')
    startedzip = True
      
  # read the data and create the connection
  data = pickle.load(gin)
  query = Query(data['sql'], data['parameters'])
  query._connect_func = data['connect_func']
  query._connect_args = data['connect_args']
  query._modified = False
  
  # close up shop and return
  if startedzip:
    gin.close() 
  if fileopened:
    query.filename = filepath
    filename.close()
  return query
  
  
def save_query(query, filename):
  '''Saves the given connection to the filename.
  
     @param query:    The query object to save.
     @type  query:    Picalo Query object
     @param filename: The filename to save to.  This can also be an open stream.
     @type  filename: str
  '''
  assert isinstance(query, Query), 'Only Picalo Query objects can be saved with this method.'
  
  # get a file pointer
  fileopened = False
  if type(filename) in types.StringTypes:
    query.filename = filename
    filename = open(filename, 'wb')
    fileopened = True
  
  # gzip it on the fly (if it is not already a GzipFile)
  openedzip = False
  gout = filename
  if not isinstance(gout, gzip.GzipFile):
    gout = gzip.GzipFile(fileobj=gout, mode='wb')
    openedzip = True
    
  # save the database connection information
  data = {
    'sql': query._sql,
    'parameters': query._parameters,
    'connect_func': query._connect_func,
    'connect_args': query._connect_args,
  }
  pickle.dump(data, gout, PICKLE_PROTOCOL)
  
  # close up shop
  gout.flush()
  if openedzip:
    gout.close()
  if fileopened:
    filename.close()
  query._modified = False
  


####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################

import inspect, types
from Error import error
from Record import Record

#################################################
###   Helper functions for Picalo

def get_current_vars(*locals):
  '''Returns the locals and globals from ALL frames top to bottom. Variables
     in inner-most frames override the outermost frames.
     
     The locals is a list of local dictionaries, with the first ones listed
     overriding any variables in the latter ones listed.  These take precendence
     over any variables found in the frames.
  '''
  # get the globals and locals from previous frames
  # this is required because functions and variables in outer frames
  # can be used in evaluate expressions
  # this is different than regular python, but it makes sense
  # in the context of cell expressions
  g = {}
  l = {}
  stack = inspect.stack()
  stack.reverse()  # so the vars from inner-most frames override the outermost frames
  for frame in stack:
    f = frame[0]
    l.update(f.f_locals)
    g.update(f.f_globals)
  # update with any locals sent in.
  for local in locals:
    l.update(local)
  return l, g
  
  
class PicaloExpression:
  '''A class that compiles an expression.  This is done so the expression
     and locals/globals are only compiled and determined one time rather
     than for each time the expression is run.  It's for efficiency.'''
  def __init__(self, expression, *locals):
    '''Constructor.  The initial_locals and initial_globals dicts override
       any local or global variables'''
    # ensure that we have a string for the expression
    if not isinstance(expression, (types.StringType, types.UnicodeType, PicaloExpression)):
      expression = unicode(expression)
    
    # save the globals, locals, and compile the code
    # we save the locals and globals dicts at creation time rather than at execution time
    self.locals, self.globals = get_current_vars(*locals)
    if isinstance(expression, PicaloExpression):
      self.expression = expression.expression  # don't wrap a PicaloExpression within a PicaloExpression
    else:
      self.expression = expression
    try:
      self.code = compile(self.expression, '<string>', 'eval')
    except Exception, e:
      self.code = compile("error('Expression is invalid: " + str(e).replace("'", "\'") + "')", '<string>', 'eval')
  
  
  def evaluate(self, locals, expression_backtrack=[]):
    '''Runs the expression given any additional locals and globals
    
       locals => A list of dictionaries containing mappings that override the values in locals
       expression_backtrack => A list of all the expression id's we've run so far, to catch circular logic.
    '''
    try:
      # ensure we haven't already done this expression in this chain
      assert not id(self) in expression_backtrack, 'Circular logic detected'
      new_expression_backtrack = [id(self)] + expression_backtrack
      # set up the locals
      l = self.locals
      if len(locals) > 0:
        l = dict(self.locals)  # make a copy so we don't modify the core dictionary
        for d in locals:
          if isinstance(d, Record):  # convenience since records almost always get passed to evaluate 
            l.update([ ( d._table.columns[j].name, d.__get_value__(j, new_expression_backtrack)) for j in range(len(d))])
          else:
            l.update(d)
      # eval the expression
      return eval(self.code, self.globals, l)
    except Exception, e:
      return error(e)
  
  
  def __repr__(self):
    return self.__str__()
    
  
  def __str__(self):
    '''Returns a string representation of this expression'''
    return '<Expression: ' + self.expression + '>'
    
    
  def __eq__(self, other):
    if isinstance(other, PicaloExpression):
      return self.expression == other.expression
    return self.expression == other
  
       
  def __ne__(self, other):
    return not self.__eq__(other)
    



####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################
#
#  THIS CLASS IS DEPRECATED. IT IS NO LONGER NEEDED NOW THAT COLUMNS
#  CAN HAVE FORMATS.  JUST USE A number FIELD AND THE FORMAT: $#,##0.00
#
#
import types, re
from Number import number

# global functions (defined in this file)
__all__ = [
  'currency',
]

RE_IMPORT = re.compile('^.*?(\d+\.\d+|\d+).*?$')

class currency(number):
  '''A simple extension to number to deliniate a currency type.  In future versions,
     we'll make this class support the currencies from around the world.'''
  def __new__(self, value=0.0):
    if value and isinstance(value, types.StringTypes):
      match = RE_IMPORT.search(value)
      if match:
        value = match.group(1)
    return number(value)
     
     



#!/usr/bin/env python
# -*- coding: windows-1251 -*-

#  Copyright (C) 2005 Roman V. Kiseliov
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#
#  3. All advertising materials mentioning features or use of this
#     software must display the following acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
#
#  4. Redistributions of any form whatsoever must retain the following
#     acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
#
#  THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
#  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
#  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
#  OF THE POSSIBILITY OF SUCH DAMAGE.


__rev_id__ = """$Id: ExcelFormulaLexer.py,v 1.4 2005/08/14 06:40:23 rvk Exp $"""


import sys
from antlr import EOF, CommonToken as Tok, TokenStream, TokenStreamException
import struct
import ExcelFormulaParser
from re import compile as recompile, match, LOCALE, UNICODE, IGNORECASE


int_const_pattern = recompile(r"\d+")
flt_const_pattern = recompile(r"\d*\.\d+(?:[Ee][+-]?\d+)?")
str_const_pattern = recompile(r'["][^"]*["]')
#range2d_pattern   = recompile(r"\$?[A-I]?[A-Z]\$?\d+:\$?[A-I]?[A-Z]\$?\d+")
ref2d_pattern     = recompile(r"\$?[A-I]?[A-Z]\$?\d+")
true_pattern      = recompile(r"TRUE", IGNORECASE)
false_pattern     = recompile(r"FALSE", IGNORECASE)
name_pattern      = recompile(r"[\.\w]+", LOCALE)

pattern_type_tuples = (
    (flt_const_pattern, ExcelFormulaParser.NUM_CONST),
    (int_const_pattern, ExcelFormulaParser.INT_CONST),
    (str_const_pattern, ExcelFormulaParser.STR_CONST),
#    (range2d_pattern  , ExcelFormulaParser.RANGE2D),
    (ref2d_pattern    , ExcelFormulaParser.REF2D),
    (true_pattern     , ExcelFormulaParser.TRUE_CONST),
    (false_pattern    , ExcelFormulaParser.FALSE_CONST),
    (name_pattern     , ExcelFormulaParser.NAME)
)


type_text_tuples = (
    (ExcelFormulaParser.NE, '<>'),
    (ExcelFormulaParser.LE, '<='),
    (ExcelFormulaParser.GE, '>='),
    (ExcelFormulaParser.EQ, '='),
    (ExcelFormulaParser.LT, '<'),
    (ExcelFormulaParser.GT, '>'),
    (ExcelFormulaParser.ADD, '+'),
    (ExcelFormulaParser.SUB, '-'),
    (ExcelFormulaParser.MUL, '*'),
    (ExcelFormulaParser.DIV, '/'),
    (ExcelFormulaParser.COLON, ':'),
    (ExcelFormulaParser.SEMICOLON, ';'),
    (ExcelFormulaParser.COMMA, ','),
    (ExcelFormulaParser.LP, '('),
    (ExcelFormulaParser.RP, ')'),
    (ExcelFormulaParser.CONCAT, '&'),
    (ExcelFormulaParser.PERCENT, '%'),
    (ExcelFormulaParser.POWER, '^')
)


class Lexer(TokenStream):
    def __init__(self, text):
        self._text = text[:]
        self._pos = 0
        self._line = 0


    def isEOF(self):
        return len(self._text) <= self._pos


    def curr_ch(self):
        return self._text[self._pos]


    def next_ch(self, n = 1):
        self._pos += n


    def is_whitespace(self):
        return self.curr_ch() in " \t\n\r\f\v"


    def match_pattern(self, pattern, toktype):
        m = pattern.match(self._text[self._pos:])
        if m:
            start_pos = self._pos + m.start(0)
            end_pos = self._pos + m.end(0)
            tt = self._text[start_pos:end_pos]
            self._pos = end_pos
            return Tok(type = toktype, text = tt, col = start_pos + 1)
        else:
            return None


    def nextToken(self):
        # skip whitespace
        while not self.isEOF() and self.is_whitespace():
            self.next_ch()
        if self.isEOF():
            return Tok(type = EOF)
        # first, try to match token with more chars
        for ptt in pattern_type_tuples:
            t = self.match_pattern(*ptt);
            if t:
                return t
        # second, we want find short tokens
        for ty, te in type_text_tuples:
            if self.curr_ch() == te:
                self.next_ch()
                return Tok(type = ty, text = te, col = self._pos)
        # at this point, smth strange is happened
        raise TokenStreamException("Unknown char %s at %u col." % (self.curr_ch(), self._pos))


if __name__ == '__main__' :
    import locale
    locale.setlocale(locale.LC_ALL, 'russian')
    try:
        for t in Lexer('1+2+3+67.8678 + " @##$$$ klhkh kljhklhkl " + .58e-678*A1:B4 - 1lkjljlkjl3535порпор'):
            print t
    except TokenStreamException, e:
        print "error:", e



####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################

import types


  
# global functions (defined in this file)
__all__ = [
  'boolean',
]


class boolean:  # btw, can't extend bool as it's a special singleton class
  '''A flag type, such as on/off, true/false, etc.  The following
     values are considered true (case is always ignored):
       - t (T)
       - true (True, TRUE, TrUe)
       - yes (Yes, YES, YeS)
       - on (ON, On, oN)
       - any integer other than 0
  '''
  def __init__(self, *args, **kargs):
    '''Creates a new boolean object'''
    if len(args) >= 1 and isinstance(args[0], types.StringTypes) and args[0].lower() in [ 't', 'true', 'yes', 'on' ]:
      self._value = True
    elif len(args) >= 1 and isinstance(args[0], types.BooleanType):
      self._value = args[0]
    elif len(args) >= 1 and isinstance(args[0], (types.IntType, types.LongType, types.FloatType)):
      self._value = (args[0] != 0)
    elif len(args) >= 1 and isinstance(args[0], boolean):
      self._value = args[0]._value
    else:
      self._value = False
    
  
  def __eq__(self, other):
    '''Returns whether this boolean value matches the other one'''
    if isinstance(other, boolean):
      return self._value == other._value
    elif isinstance(other, types.BooleanType):
      return self._value == other
    else:
      return self._value == boolean(other)._value
      
      
  def __repr__(self):
    return self._value and 'True' or 'False'
    
    
